import magicmind.python.runtime as mm
import argparse
import numpy as np
import paddle
import paddle.vision.transforms as T
import cv2
import torch
import math
import sys
import os
import logging
from  paddle_utils import multiclass_nms
from metrics import Metric, COCOMetric

transform = T.ToTensor()
normalize = T.Normalize(mean=[0.485, 0.456, 0.406],
        std=[0.229, 0.224, 0.225],
        data_format='CHW')
 
parser = argparse.ArgumentParser()
parser.add_argument("--device_id", "--device_id", type = int, default = 0, help = "device_id")
parser.add_argument("--magicmind_model", "--magicmind_model",type=str,default="../data/models/yolov5_pytorch_model_qint8_mixed_float16_true_1")
parser.add_argument("--image_dir", "--image_dir",  type=str, default="../../../../datasets/coco/val2017", help="coco val datasets")
parser.add_argument("--image_num", "--image_num",  type=int, default=10, help="image number")
parser.add_argument("--file_list", "--file_list",  type=str, default="../../../../datasets/coco/file_list_5000.txt", help="coco file list")
parser.add_argument("--label_path", "--label_path", type=str, default="../../../../datasets/coco/coco.names")
parser.add_argument("--output_dir", "--output_dir", type=str, default="../data/images/output")
parser.add_argument('--input_width', dest = 'input_width', default = 608, type = int, help = 'model input width')
parser.add_argument('--input_height', dest = 'input_height', default = 608, type = int, help = 'model input height')
parser.add_argument('--batch', dest = 'batch', default = 1, type = int, help = 'model input batch')
parser.add_argument("--save_img", "--save_img", type=bool, default=False)
 
def coco_dataset(
    file_list_txt="../../../../datasets/coco/file_list_5000.txt",
    image_dir="../../../../datasets/coco/val2017",
    count=-1
):
    with open(file_list_txt, "r") as f:
        lines = f.readlines()
    logging.info("%d pictures will be read." % len(lines))
    current_count = 0
    for line in lines:
        image_name = line.strip()
        image_path = os.path.join(image_dir, image_name)
        img = cv2.imread(image_path)
        yield img, image_path
        current_count += 1
        if current_count >= count and count != -1:
            break
 
def letterbox(img, dst_shape):
    src_h, src_w = img.shape[0], img.shape[1]
    dst_h, dst_w = dst_shape
    ratio = (dst_h / src_h, dst_w / src_w)
    img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
    img = cv2.resize(img, (dst_shape[0], dst_shape[1]), interpolation=cv2.INTER_CUBIC)
    img = transform(img)
    img = normalize(img)
    img = paddle.unsqueeze(img, axis=0).numpy()
    return img, ratio
 
if __name__ == "__main__":
    args = parser.parse_args()
    if not os.path.exists(args.magicmind_model):
        print("please generate magicmind model first!!!")
        exit()
    anno_file = os.environ.get('COCO_DATASETS_PATH') + "/annotations/instances_val2017.json"
    clsid2catid = {0: 1, 1: 2, 2: 3, 3: 4, 4: 5, 5: 6, 6: 7, 7: 8, 8: 9, 9: 10, 10: 11, 11: 13, 12: 14, 13: 15, 14: 16, 15: 17, 16: 18, 17: 19, 18: 20, 19: 21, 20: 22, 21: 23, 22: 24, 23: 25, 24: 27, 25: 28, 26: 31, 27: 32, 28: 33, 29: 34, 30: 35, 31: 36, 32: 37, 33: 38, 34: 39, 35: 40, 36: 41, 37: 42, 38: 43, 39: 44, 40: 46, 41: 47, 42: 48, 43: 49, 44: 50, 45: 51, 46: 52, 47: 53, 48: 54, 49: 55, 50: 56, 51: 57, 52: 58, 53: 59, 54: 60, 55: 61, 56: 62, 57: 63, 58: 64, 59: 65, 60: 67, 61: 70, 62: 72, 63: 73, 64: 74, 65: 75, 66: 76, 67: 77, 68: 78, 69: 79, 70: 80, 71: 81, 72: 82, 73: 84, 74: 85, 75: 86, 76: 87, 77: 88, 78: 89, 79: 90}
    classwise = False
    output_eval = args.output_dir
    bias = 0
    IouType = 'bbox'
    _metrics = [
            COCOMetric(
                anno_file=anno_file,
                clsid2catid=clsid2catid,
                classwise=classwise,
                output_eval=output_eval,
                bias=bias,
                IouType=IouType)
                ]

    model = mm.Model()
    model.deserialize_from_file(args.magicmind_model)
 
    img_size = [args.input_width, args.input_height]
    with mm.System() as mm_sys:
        dev_count = mm_sys.device_count()
        print("Device count: ", dev_count)
        assert args.device_id < dev_count
        # 打开MLU设备
        dev = mm.Device()
        dev.id = args.device_id
        assert dev.active().ok()
        # 创建Engine
        econfig = mm.Model.EngineConfig()
        econfig.device_type = "MLU"
        engine = model.create_i_engine(econfig)
        assert engine != None, "Failed to create engine"
        # 创建Context
        context = engine.create_i_context()
        assert context != None
        # 创建MLU任务队列
        queue = dev.create_queue()
        assert queue != None
        # 创建输入tensor, 输出tensor
        mm_inputs = context.create_inputs()
        mm_outputs = []
        inputs = [None] * 3
        nms_kwargs={'score_threshold': 0.01, 'nms_top_k': 1000, 'keep_top_k': 100, 'nms_threshold': 0.45, 'normalized': True, 'nms_eta': 1.0, 'return_index': False, 'return_rois_num': True, 'background_label': 80}
        dataset = coco_dataset(file_list_txt = args.file_list, image_dir = args.image_dir, count = args.image_num)
        data = {}
        print("Start run ...")
        for img, img_path in dataset:
            img_name = os.path.splitext(img_path.split("/")[-1])[0]
            print("Inference img : ", img_name)
            # 准备输入数据
            show_img = img
            img, ratio = letterbox(img, img_size)
            inputs[0] = np.array([[img_size[0], img_size[1]]]).astype(np.float32)
            inputs[1] = img
            inputs[2] = np.array([[ratio[0], ratio[1]]]).astype(np.float32)
            for idx, i in enumerate(inputs):
                mm_inputs[idx].from_numpy(inputs[idx])
                mm_inputs[idx].to(dev)
            # 向MLU下发任务
            assert context.enqueue(mm_inputs, mm_outputs, queue).ok()
            # 等待任务执行完成
            assert queue.sync().ok()
            # 处理输出数据
            bbox = mm_outputs[0].asnumpy()
            score = mm_outputs[1].asnumpy()
            bboxes = paddle.to_tensor(bbox)
            scores = paddle.to_tensor(score)
            bbox_pred, bbox_num, _ = multiclass_nms(bboxes, scores, **nms_kwargs)
            bboxes = bbox_pred.numpy()
            bbox_num = bbox_num.numpy()
            outs ={}
            outs['bbox'] = paddle.to_tensor(bbox_pred)
            outs['bbox_num'] = paddle.to_tensor(bbox_num)
            #outs = self.model(data)
            image_id = img_name.split(".")[0]
            data['im_id'] = paddle.to_tensor([[int(image_id)]])
            for metric in _metrics:
                    metric.update(data, outs)
        for metric in _metrics:
            metric.accumulate()
            metric.log()
        for metric in _metrics:
            metric.reset()




import json
import argparse
import magicmind.python.runtime as mm
from framework_parser import TensorFlowParser
from build_param import get_argparser 
from model_process import (
    extract_params,
    config_network,
    get_builder_config,
    build_and_serialize,
)
from logger import Logger
log = Logger()
import tensorflow as tf
log.info(tf.__version__)
log.info("tf version is:tf{}".format(tf.__version__[0]))


def get_network(args):
    parser = TensorFlowParser(args)
    network = parser.parse()
    input_dims = mm.Dims((args.input_dims[0]))
    assert network.get_input(0).set_dimension(input_dims).ok()
    assert network.get_input(1).set_dimension(input_dims).ok()
    return network

def get_args():
    arg_parser = get_argparser()
    return arg_parser.parse_args()

def main():
    args = get_args()
    assert args.precision in ['force_float16', 'force_float32']
    network = get_network(args)
    config_args = extract_params("MODEL_CONFIG", args)
    config_network(network, config_args)
    builder_args = extract_params("MODEL_BUILDER", args)
    build_config = get_builder_config(builder_args)

    # generate model
    log.info('build model...')
    builder = mm.Builder()
    model = builder.build_model("bert_tensorflow_model", network, build_config)
    assert model is not None
    assert model.serialize_to_file(args.magicmind_model).ok()
    log.info("Generate model done, model save to %s" % args.magicmind_model) 
if __name__ == "__main__":
    main()



""" from https://github.com/keithito/tacotron """
import re
from . import cleaners
from .symbols import symbols


# Mappings from symbol to numeric ID and vice versa:
_symbol_to_id = {s: i for i, s in enumerate(symbols)}
_id_to_symbol = {i: s for i, s in enumerate(symbols)}

# Regular expression matching text enclosed in curly braces:
_curly_re = re.compile(r'(.*?)\{(.+?)\}(.*)')


def text_to_sequence(text, cleaner_names):
  '''Converts a string of text to a sequence of IDs corresponding to the symbols in the text.

    The text can optionally have ARPAbet sequences enclosed in curly braces embedded
    in it. For example, "Turn left on {HH AW1 S S T AH0 N} Street."

    Args:
      text: string to convert to a sequence
      cleaner_names: names of the cleaner functions to run the text through

    Returns:
      List of integers corresponding to the symbols in the text
  '''
  sequence = []

  # Check for curly braces and treat their contents as ARPAbet:
  while len(text):
    m = _curly_re.match(text)
    if not m:
      sequence += _symbols_to_sequence(_clean_text(text, cleaner_names))
      break
    sequence += _symbols_to_sequence(_clean_text(m.group(1), cleaner_names))
    sequence += _arpabet_to_sequence(m.group(2))
    text = m.group(3)

  return sequence


def sequence_to_text(sequence):
  '''Converts a sequence of IDs back to a string'''
  result = ''
  for symbol_id in sequence:
    if symbol_id in _id_to_symbol:
      s = _id_to_symbol[symbol_id]
      # Enclose ARPAbet back in curly braces:
      if len(s) > 1 and s[0] == '@':
        s = '{%s}' % s[1:]
      result += s
  return result.replace('}{', ' ')


def _clean_text(text, cleaner_names):
  for name in cleaner_names:
    cleaner = getattr(cleaners, name)
    if not cleaner:
      raise Exception('Unknown cleaner: %s' % name)
    text = cleaner(text)
  return text


def _symbols_to_sequence(symbols):
  return [_symbol_to_id[s] for s in symbols if _should_keep_symbol(s)]


def _arpabet_to_sequence(text):
  return _symbols_to_sequence(['@' + s for s in text.split()])


def _should_keep_symbol(s):
  return s in _symbol_to_id and s is not '_' and s is not '~'



import os
from skimage import io, transform
import torch
import torchvision
import torch
import sys
sys.path.append("./U-2-Net")
from model import U2NET # full size version 173.6 MB
from model import U2NETP # small version u2net 4.7 MB
#import torch_mlu.core.mlu_model as ct

def main():

    # --------- 1. get image path and name ---------
    model_name='u2net'#u2net
    MODEL_PATH = os.getenv("MODEL_PATH")
    model_dir = os.path.join(MODEL_PATH, model_name + '.pth')

    if(model_name=='u2net'):
        print("...load U2NET---173.6 MB")
        net = U2NET(3,1)
    elif(model_name=='u2netp'):
        print("...load U2NEP---4.7 MB")
        net = U2NETP(3,1)

    net.load_state_dict(torch.load(model_dir, map_location='cpu'))
    net.eval()
    trace_input = torch.randn(1, 3, 320, 320, dtype = torch.float)
    trace_model = torch.jit.trace(net, (trace_input))
    torch.jit.save(trace_model, MODEL_PATH + os.sep + model_name + '.pt')
    exit()

if __name__ == "__main__":
    main()



import torch
from torchvision.datasets import CIFAR100
import sys
import os

if __name__=="__main__":
    CIFAR100_DATASETS_PATH = sys.argv[1]
    # download dataset
    CIFAR100(root=os.path.expanduser(CIFAR100_DATASETS_PATH), download=True, train=False)

    
    


import numpy as np
import os
import timeit
from sklearn.metrics import roc_curve, auc
import pandas as pd
from tqdm import tqdm
from utils import Record

IJB_DATASETS_PATH = os.environ.get("IJB_DATASETS_PATH")

def load_feature(features_dir, img_id):
    feature_file = features_dir + '/' + str(img_id) + '.feature'
    feature = np.fromfile(feature_file, dtype=float, count=-1, sep=' ')
    # faceness_score = feature[512]
    feature = feature[0:512] * feature[512]
    return feature

def load_features(features_dir, img_ids):
    from multiprocessing import Pool, cpu_count
    from functools import partial
    import math
    process_num = int(math.ceil(cpu_count() * 0.9))
    func = partial(load_feature, features_dir)
    with Pool(process_num) as p:
        features = list(tqdm(p.imap(func, img_ids), total=len(img_ids), desc='load features'))
    features = np.array(features, dtype=float)
    print('features shape:', features.shape)
    return features

def summary(features_dir, template_media_list_path,
            read_template_pair_list_path):
    templates, medias = read_template_media_list(template_media_list_path)
    p1, p2, label = read_template_pair_list(read_template_pair_list_path)
    
    with open(template_media_list_path, "r") as f:
        lines = f.readlines()
        img_ids = [int(id.split(" ")[0].split(".")[0]) for id in lines]
     
    features = load_features(features_dir, img_ids)

    template_norm_feats, unique_templates = image2template_feature(features, templates, medias)

    score = verification(template_norm_feats, unique_templates, p1, p2)
    methods = np.array(["ijbc"])
    scores = dict(zip(methods, [score]))

    x_labels = [10**-6, 10**-5, 10**-4, 10**-3, 10**-2, 10**-1]
    y_labels = []

    for method in methods:
        fpr, tpr, _ = roc_curve(label, scores[method])
        roc_auc = auc(fpr, tpr)
        fpr = np.flipud(fpr)
        tpr = np.flipud(tpr)  # select largest tpr at same fpr
        for fpr_iter in np.arange(len(x_labels)):
            _, min_index = min(
                list(zip(abs(fpr - x_labels[fpr_iter]), range(len(fpr)))))
            y_labels.append('%.5f' % (tpr[min_index] * 100))
    return {"1e-5": y_labels[1], "1e-4": y_labels[2]}, y_labels[1], y_labels[2]


def read_template_pair_list(path):
    import pandas as pd
    pairs = pd.read_csv(path, sep=' ', header=None).values
    t1 = pairs[:, 0].astype(np.int)
    t2 = pairs[:, 1].astype(np.int)
    label = pairs[:, 2].astype(np.int)
    return t1, t2, label


def read_template_media_list(path):
    import pandas as pd
    ijb_meta = pd.read_csv(path, sep=' ', header=None).values
    templates = ijb_meta[:, 1].astype(np.int)
    medias = ijb_meta[:, 2].astype(np.int)
    return templates, medias


def image2template_feature(img_feats=None, templates=None, medias=None):
    from sklearn import preprocessing
    # ==========================================================
    # 1. face image feature l2 normalization. img_feats:[number_image x feats_dim]
    # 2. compute media feature.
    # 3. compute template feature.
    # ==========================================================
    unique_templates = np.unique(templates)
    template_feats = np.zeros((len(unique_templates), img_feats.shape[1]))

    for count_template, uqt in enumerate(unique_templates):

        (ind_t, ) = np.where(templates == uqt)
        # max_value = np.max(ind_t)
        #(ind_t, ) = np.where(ind_t < img_feats.shape[0])
        face_norm_feats = img_feats[ind_t]
        face_medias = medias[ind_t]
        unique_medias, unique_media_counts = np.unique(face_medias,
                                                       return_counts=True)
        media_norm_feats = []
        for u, ct in zip(unique_medias, unique_media_counts):
            (ind_m, ) = np.where(face_medias == u)
            if ct == 1:
                media_norm_feats += [face_norm_feats[ind_m]]
            else:  # image features from the same video will be aggregated into one feature
                media_norm_feats += [
                    np.mean(face_norm_feats[ind_m], axis=0, keepdims=True)
                ]
        media_norm_feats = np.array(media_norm_feats)
        # media_norm_feats = media_norm_feats / np.sqrt(np.sum(media_norm_feats ** 2, -1, keepdims=True))
        template_feats[count_template] = np.sum(media_norm_feats, axis=0)
        if count_template % 2000 == 0:
            print('Finish Calculating {} template features.'.format(
                count_template))
    # template_norm_feats = template_feats / np.sqrt(np.sum(template_feats ** 2, -1, keepdims=True))
    template_norm_feats = preprocessing.normalize(template_feats)
    return template_norm_feats, unique_templates


def verification(template_norm_feats=None,
                 unique_templates=None,
                 p1=None,
                 p2=None):
    # ==========================================================
    #         Compute set-to-set Similarity Score.
    # ==========================================================
    template2id = np.zeros((max(unique_templates) + 1, 1), dtype=int)
    for count_template, uqt in enumerate(unique_templates):
        template2id[uqt] = count_template

    score = np.zeros((len(p1), ))  # save cosine distance between pairs

    total_pairs = np.array(range(len(p1)))
    batchsize = 100000  # small batchsize instead of all pairs in one batch due to the memory limiation
    sublists = [
        total_pairs[i:i + batchsize] for i in range(0, len(p1), batchsize)
    ]
    total_sublists = len(sublists)
    for c, s in enumerate(sublists):
        feat1 = template_norm_feats[template2id[p1[s]]]
        feat2 = template_norm_feats[template2id[p2[s]]]
        similarity_score = np.sum(feat1 * feat2, -1)
        score[s] = similarity_score.flatten()
        if c % 10 == 0:
            print('Finish {}/{} pairs.'.format(c, total_sublists))
    return score

def main():
    import argparse
    args = argparse.ArgumentParser(description='evaluate IJBC')
    args.add_argument('--features_dir', default = 'output_features',
            required = True, type = str, help = 'features directory')
    args.add_argument('--output_file', default = 'output_file',
            required = True, type = str, help = 'result file')
    args.add_argument('--face_tid_mid_file', default = str(IJB_DATASETS_PATH) + '/IJBC/meta/ijbc_face_tid_mid.txt',
            required = False, type = str, help = 'result file')
    args.add_argument('--template_pair_label_file', default = str(IJB_DATASETS_PATH) + '/IJBC/meta/ijbc_template_pair_label.txt',
            required = False, type = str, help = 'result file')
    args = args.parse_args()
    res, res_1e5, res_1e4 = summary(args.features_dir,  args.face_tid_mid_file, args.template_pair_label_file)
    print(res)
    result_file = Record(args.output_file)
    result_file.write("IJB-C(1E-5): %s"%(res_1e5), False)
    result_file.write("IJB-C(1E-4): %s"%(res_1e4), False)

if __name__ == "__main__":
    main()




import sys
import os
PROJ_ROOT_PATH = os.environ.get("PROJ_ROOT_PATH")
sys.path.append('Pytorch_Retinaface')
import torch
from data import cfg_mnet, cfg_re50
from models.retinaface import RetinaFace

import argparse

cfgs = dict([('mobile0.25', cfg_mnet), ('resnet50', cfg_re50)])

parser = argparse.ArgumentParser()
parser.add_argument("-b", "--batch_size", type=int, default=1, help="batch_size")
parser.add_argument("--cfg", type=str, default='mobile0.25', help="cfg file, can be \'mobile0.25\' or \'resnet50\'")
parser.add_argument('--weights', type=str, default= str(PROJ_ROOT_PATH) + '/data/models/Resnet50_Final.pth', help='weights path')
parser.add_argument('--imgsz', '--img', '--img-size', nargs='+', type=int, default=[672, 1024], help='image (h, w)')
parser.add_argument("--traced_pt", type=str, default= str(PROJ_ROOT_PATH) + '/data/models/retinaface_traced.pt', help="traced pt file")

# 参考test_widerface.py
def remove_prefix(state_dict, prefix):
    print('remove prefix \'{}\''.format(prefix))
    f = lambda x: x.split(prefix, 1)[-1] if x.startswith(prefix) else x
    return {f(key): value for key, value in state_dict.items()}

if __name__ == "__main__":
    args = parser.parse_args()
    cfgs[args.cfg]['pretrain']=False
    model = RetinaFace(cfg=cfgs[args.cfg], phase='test')
    pretrained_dict = torch.load(args.weights, map_location='cpu')
    if "state_dict" in pretrained_dict.keys():
        pretrained_dict = remove_prefix(pretrained_dict['state_dict'], 'module.')
    else:
        pretrained_dict = remove_prefix(pretrained_dict, 'module.')
    model.load_state_dict(pretrained_dict, strict=False)
    model.eval()
    rand_input = torch.randn(args.batch_size, 3, args.imgsz[0], args.imgsz[1]).float()
    traced_model = torch.jit.trace(model, rand_input)
    torch.jit.save(traced_model, args.traced_pt)




import os
from re import S
from PIL import Image
import cv2
from cv2 import resize
import numpy as np
import logging
import argparse
import sys
sys.path.append(os.path.join(os.getenv("MAGICMIND_CLOUD"), "test"))
VOC_DATASETS_PATH = os.environ.get("VOC2012_DATASETS_PATH")
PROJ_ROOT_PATH = os.environ.get("PROJ_ROOT_PATH")

def voc_dataset(file_list, image_file_path, count):
    with open(file_list, "r") as f:
        lines = f.readlines()
    logging.info("%d pictures will be read." % len(lines))
    current_count = 0
    for line in lines:
        image_name = line.replace("\n", "")
        image_path = os.path.join(image_file_path, image_name + ".jpg")
        img = Image.open(image_path)
        yield img
        current_count += 1
        if current_count > count and count != -1:
            break
class Record:
    def __init__(self, filename):
        self.file = open(os.path.join(filename), "w")

    def write(self, line, _print = False):
        self.file.write(line + "\n")
        if _print:
            print(line)

LABEL_NAMES = np.asarray([
    'background', 'aeroplane', 'bicycle', 'bird', 'boat', 'bottle', 'bus',
    'car', 'cat', 'chair', 'cow', 'diningtable', 'dog', 'horse', 'motorbike',
    'person', 'pottedplant', 'sheep', 'sofa', 'train', 'tv'
])

def per_class_iu(hist):
    return np.diag(hist) / (hist.sum(1) + hist.sum(0) - np.diag(hist))
 
def fast_hist(a, b, n):
    k = (a >= 0) & (a < n)
    return np.bincount(n * a[k].astype(int) + b[k], minlength=n ** 2).reshape(n, n)

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('--image_num', type = int, default = 1000, help = 'image number')
    parser.add_argument('--gt_dir', type = str, default = str(VOC_DATASETS_PATH) + '/VOCdevkit/VOC2012/SegmentationClass', help ='directory which stores VOC Segmentation val gt images')
    parser.add_argument('--pred_dir', type = str, default = str(PROJ_ROOT_PATH) + '/data/output/', help ='directory which stores VOC Segmentation val pred images')
    parser.add_argument('--file_list', type = str, default = str(VOC_DATASETS_PATH) + '/VOCdevkit/VOC2012/ImageSets/Segmentation/val.txt', help='file list which uses to compute mIOU')
    parser.add_argument("--language", dest = "language", help = "language which used to infer model", default = "infer_python", type = str)
    args = parser.parse_args()
    image_path = args.pred_dir
    gt_path = args.gt_dir
    val_txt = args.file_list

    num_classes = 21
    name_classes = ["backgroud", "aeroplane", "bicycle", "bird", "boat", "bottle", "bus", "car",
                    "cat", "chair", "cow", "diningtable", "dog", "horse", "motorbike", "person",
                    "pottedplant", "sheep", "sofa", "train", "tvmonitor"]
    hist = np.zeros((num_classes, num_classes))
    
    with open(val_txt, "r") as f:
        lines = f.readlines()
    #record = Record("eval_result.txt")
    count = 0
    for line in lines:
        line = line.strip()
        image_file = os.path.join(image_path, line + ".png")

        # load gt
        gt_file_path = os.path.join(gt_path, line + ".png")
        label_org = Image.open(gt_file_path)
        #print(image_file)
        pred_image = Image.open(image_file)
        pred_image = pred_image.resize(label_org.size)
        pred_p = pred_image.quantize(palette = label_org) # convert to P mode

        label = np.array(label_org)
        pred = np.array(pred_p)
        hist += fast_hist(label.flatten(), pred.flatten(), num_classes)
        count += 1
        if count > args.image_num -1:
            break
    mIoUs = per_class_iu(hist)
    for ind_class in range(num_classes):
        print('===>' + name_classes[ind_class] + ':\t' + str(round(mIoUs[ind_class] * 100, 2)), True)
        #record.write('===>' + name_classes[ind_class] + ':\t' + str(round(mIoUs[ind_class] * 100, 2)), True)
    #record.write('mIOU:' + str(round(np.nanmean(mIoUs) * 100, 2)), True)
    print('mIOU:' + str(round(np.nanmean(mIoUs) * 100, 2)))



import numpy as np
import magicmind.python.runtime as mm
import cv2
import os
import glob
import math
from PIL import Image
from logger import Logger

log = Logger()


def letterbox(img, dst_shape):
    src_h, src_w = img.shape[0], img.shape[1]
    dst_h, dst_w = dst_shape
    ratio = min(dst_h / src_h, dst_w / src_w)
    unpad_h, unpad_w = int(math.floor(src_h * ratio)), int(math.floor(src_w * ratio))
    if ratio != 1:
        interp = cv2.INTER_AREA if ratio < 1 else cv2.INTER_LINEAR
        img = cv2.resize(img, (unpad_w, unpad_h), interpolation=interp)
    # padding
    pad_t = int(math.floor((dst_h - unpad_h) / 2))
    pad_b = dst_h - unpad_h - pad_t
    pad_l = int(math.floor((dst_w - unpad_w) / 2))
    pad_r = dst_w - unpad_w - pad_l
    img = cv2.copyMakeBorder(
        img, pad_t, pad_b, pad_l, pad_r, cv2.BORDER_CONSTANT, value=(114, 114, 114)
    )
    return img, ratio


def image_preporcess(image, target_size, gt_boxes=None):
    image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB).astype(np.float32)

    ih, iw = target_size
    h, w, _ = image.shape

    scale = min(iw / w, ih / h)
    nw, nh = int(scale * w), int(scale * h)
    image_resized = cv2.resize(image, (nw, nh))

    image_paded = np.full(shape=[ih, iw, 3], fill_value=128.0)
    dw, dh = (iw - nw) // 2, (ih - nh) // 2
    image_paded[dh : nh + dh, dw : nw + dw, :] = image_resized
    image_paded = image_paded / 255.0

    if gt_boxes is None:
        return image_paded

    else:
        gt_boxes[:, [0, 2]] = gt_boxes[:, [0, 2]] * scale + dw
        gt_boxes[:, [1, 3]] = gt_boxes[:, [1, 3]] * scale + dh
        return image_paded, gt_boxes


def preprocess_image(img, input_size) -> np.ndarray:
    original_image = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
    image_data = image_preporcess(np.copy(original_image), [input_size, input_size])
    image_data = image_data.astype(np.float32)
    return image_data


class CalibData(mm.CalibDataInterface):
    def __init__(self, shape: mm.Dims, max_samples: int, img_dir: str):
        super().__init__()
        assert os.path.isdir(img_dir)
        self.data_paths_ = glob.glob(img_dir + "/*.jpg")
        self.shape_ = shape
        self.max_samples_ = min(max_samples, len(self.data_paths_))
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        # h and w
        self.dst_shape_ = (self.shape_.GetDimValue(1), self.shape_.GetDimValue(2))

    def get_shape(self):
        return self.shape_

    def get_data_type(self):
        return mm.DataType.FLOAT32

    def get_sample(self):
        return self.cur_sample_

    def preprocess_images(self, data_begin: int, data_end: int) -> np.ndarray:
        imgs = []
        for i in range(data_begin, data_end):
            img = cv2.imread(self.data_paths_[i])
            img = preprocess_image(img, self.dst_shape_[0])
            imgs.append(img[np.newaxis, :])
        # batch and normalize
        return np.ascontiguousarray(np.concatenate(tuple(imgs), axis=0))

    def next(self):
        batch_size = self.shape_.GetDimValue(0)
        data_begin = self.cur_data_index_
        data_end = data_begin + batch_size
        if data_end > self.max_samples_:
            return mm.Status(mm.Code.OUT_OF_RANGE, "Data end reached")
        self.cur_sample_ = self.preprocess_images(data_begin, data_end)
        self.cur_data_index_ = data_end
        return mm.Status.OK()

    def reset(self):
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        return mm.Status.OK()



import magicmind.python.runtime as mm
import numpy as np
import glob
import os 
from PIL import Image
from torchvision import transforms

resize_h, resize_w = (256, 256)
crop_h, crop_w = (224, 224)
mean = [0.485, 0.456, 0.406]
std = [0.229, 0.224, 0.225]

def pre_process(input_image, transpose):
    normalize = transforms.Normalize(mean, std)
    preprocess = transforms.Compose(
        [
            transforms.Resize(resize_h),
            transforms.CenterCrop(crop_h),
            transforms.ToTensor(),
            normalize,
        ]
    )
    input_tensor = preprocess(input_image)
    input_numpy = input_tensor.numpy()
    if transpose:
        input_numpy = np.transpose(input_numpy, (1, 2, 0))
    return input_numpy

class CalibData(mm.CalibDataInterface):
    def __init__(self, shape: mm.Dims, max_samples: int, img_dir: str):
        super().__init__()
        assert os.path.isdir(img_dir)
        self.data_paths_ = glob.glob(img_dir + '/*.jpg')
        t = glob.glob(img_dir + '/*.JPEG')
        self.data_paths_ += t
        self.shape_ = shape
        self.max_samples_ = min(max_samples, len(self.data_paths_))
        self.cur_sample_ = None
        self.cur_data_index_ = 0

    def get_shape(self):
        return self.shape_

    def get_data_type(self):
        return mm.DataType.FLOAT32

    def get_sample(self):
        return self.cur_sample_
    
    def preprocess_images(self, data_begin: int, data_end: int) -> np.ndarray:
        imgs = []
        for i in range(data_begin, data_end):
            img = Image.open(self.data_paths_[i])
            if img.mode != "RGB":
                img = img.convert("RGB")
            img = pre_process(img,transpose = True)
            imgs.append(np.ascontiguousarray(img)[np.newaxis,:])
        # batch
        return np.ascontiguousarray(np.concatenate(tuple(imgs), axis=0))

    def next(self):
        batch_size = self.shape_.GetDimValue(0)
        data_begin = self.cur_data_index_
        data_end = data_begin + batch_size
        if data_end > self.max_samples_:
            return mm.Status(mm.Code.OUT_OF_RANGE, "Data end reached")
        self.cur_sample_ = self.preprocess_images(data_begin, data_end)
        self.cur_data_index_ = data_end
        return mm.Status.OK()

    def reset(self):
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        return mm.Status.OK()



import numpy as np
import argparse
from utils import Record
def get_args():
    parser = argparse.ArgumentParser(description='Calculate the TOP1/5 of imagenet dataset')

    parser.add_argument("--result_label_file", dest = 'result_label_file', help = "result label txt",
                        default = "/mm_ws/proj/modelzoo/magicmind_cloud/buildin/cv/classification/resnet50_onnx/data/images/output/eval_labels.txt", type = str)
        
    parser.add_argument("--result_1_file", dest = 'result_1_file', help = "top1 accuracy result file path",
                        default = "/mm_ws/proj/modelzoo/magicmind_cloud/buildin/cv/classification/resnet50_onnx/data/images/output/eval_result_1.txt", type = str)

    parser.add_argument("--result_5_file", dest = 'result_5_file', help = "top5 accuracy result file path",
                        default = "/mm_ws/proj/modelzoo/magicmind_cloud/buildin/cv/classification/resnet50_onnx/data/images/output/eval_result_5.txt", type = str)
    parser.add_argument("--top1andtop5_file", dest = 'top1andtop5_file', help = "top1 and top5 accuracy", default = "/mm_ws/proj/modelzoo/magicmind_cloud/buildin/cv/classification/resnet50_onnx/data/images/output/eval_result.txt")

    return parser.parse_args()

if __name__ == "__main__":
    args = get_args()
        
    top1_count = 0
    top5_count = 0
    total_count = 0
    
    with open(args.result_1_file, "r") as r1:
        top1_results = r1.readlines()
    with open(args.result_5_file, "r") as r5:
        top5_results = r5.readlines()
    with open(args.result_label_file, "r") as l:
        labels= l.readlines()

    for i, label in enumerate(labels):
        if label == top1_results[i]:
            top1_count += 1
        for top5_result in top5_results[i*5:(i+1)*5]:
            if label == top5_result:
                top5_count += 1
        total_count += 1
       
    top1 = float(top1_count) / float(total_count)
    top5 = float(top5_count) / float(total_count)
    print("top1: ", top1)
    print("top5: ", top5)
    result_top1andtop5 = Record(args.top1andtop5_file)
    result_top1andtop5.write("top1 accuracy: %f"%(top1), False)
    result_top1andtop5.write("top5 accuracy: %f"%(top5), False)



from magicmind.python.runtime import DataType, ModelKind, Network
from magicmind.python.runtime.parser import Parser

from logger import Logger

log = Logger()


class TensorFlowParser(object):
    def __init__(self, args):
        self.parser = Parser(ModelKind.kTensorflow)
        self.args = args

    def parse_model(self, network: Network) -> Network:
        """Parse model.

        Args:
            network(Network): Out parameter, the parsed network.

        Returns:
            - Returns a parsed network.
        """
        self.parser.set_model_param("tf-model-type", "tf-graphdef-file")
        self.parser.set_model_param("tf-graphdef-inputs", self.args.input_names)
        self.parser.set_model_param("tf-graphdef-outputs", self.args.output_names)
        self.parser.set_model_param("tf-infer-shape", True)
        self.parser.parse(network, self.args.tf_pb)
        return network

    def parse(self) -> Network:
        """Parse framework model."""
        network = Network()
        self.parse_model(network)
        log.info("TensorFlowModelParser: input model was parsed successfully.")
        return network


class PytorchParser(object):
    def __init__(self, args):
        self.parser = Parser(ModelKind.kPytorch)
        self.args = args

    def parse_model(self, network: Network) -> Network:
        """Parse model.

        Args:
            network(Network): Out parameter, the parsed network.

        Returns:
            - Returns a parsed network.
        """
        DATATYPE_TO_MM = {
            "INT8": DataType.INT8,
            "INT16": DataType.INT16,
            "INT32": DataType.INT32,
            "UINT8": DataType.UINT8,
            "UINT16": DataType.UINT16,
            "UINT32": DataType.UINT32,
            "HALF": DataType.FLOAT16,
            "FLOAT": DataType.FLOAT32,
            "BOOL": DataType.BOOL,
            "QINT8": DataType.QINT8,
            "QINT16": DataType.QINT16,
        }

        # there may have multiple inputs
        # dt = [self.args["pt_input_dtypes"]] if isinstance(self.args["pt_input_dtypes"],
        #                                                  str) else self.args["pt_input_dtypes"]
        dt = (
            [self.args.pt_input_dtypes]
            if isinstance(self.args.pt_input_dtypes, str)
            else self.args.pt_input_dtypes
        )
        self.parser.set_model_param(
            "pytorch-input-dtypes", [DATATYPE_TO_MM[t] for t in dt]
        )
        self.parser.parse(network, self.args.pytorch_pt)
        # self.parser.parse(network, self.args["pytorch_pt"])
        return network

    def parse(self) -> Network:
        """Parse framework model."""
        network = Network()
        self.parse_model(network)
        log.info("PytorchModelParser: input model was parsed successfully.")
        return network


class CaffeParser(object):
    def __init__(self, args):
        self.parser = Parser(ModelKind.kCaffe)
        self.args = args

    def parse_model(self, network: Network) -> Network:
        """Parse model.

        Args:
            network(Network): Out parameter, the parsed network.

        Returns:
            - Returns a parsed network.
        """
        self.parser.parse(network, self.args.caffemodel, self.args.prototxt)
        return network

    def parse(self) -> Network:
        """Parse framework model."""
        network = Network()
        self.parse_model(network)
        log.info("CaffeModelParser: input model was parsed successfully.")
        return network


class OnnxParser(object):
    def __init__(self, args):
        self.parser = Parser(ModelKind.kOnnx)
        self.args = args

    def parse_model(self, network: Network) -> Network:
        """Parse model.

        Args:
            network(Network): Out parameter, the parsed network.

        Returns:
            - Returns a parsed network.
        """
        self.parser.parse(network, self.args.onnx)
        return network

    def parse(self) -> Network:
        """Parse framework model."""
        network = Network()
        self.parse_model(network)
        log.info("OnnxModelParser: input model was parsed successfully.")
        return network



# Copyright (c) 2020 PaddlePaddle Authors. All Rights Reserved. 
#   
# Licensed under the Apache License, Version 2.0 (the "License");   
# you may not use this file except in compliance with the License.  
# You may obtain a copy of the License at   
#   
#     http://www.apache.org/licenses/LICENSE-2.0    
#   
# Unless required by applicable law or agreed to in writing, software   
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
# See the License for the specific language governing permissions and   
# limitations under the License.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
import sys
import json
import paddle
import numpy as np
import typing
from collections import defaultdict
from pathlib import Path

from json_results import get_det_res, get_det_poly_res, get_seg_res, get_solov2_segm_res, get_keypoint_res
from category import get_categories

from logger import setup_logger
logger = setup_logger(__name__)

__all__ = [
    'Metric', 'COCOMetric'
]

def get_infer_results(outs, catid, bias=0):
    """
    Get result at the stage of inference.
    The output format is dictionary containing bbox or mask result.

    For example, bbox result is a list and each element contains
    image_id, category_id, bbox and score.
    """
    if outs is None or len(outs) == 0:
        raise ValueError(
            'The number of valid detection result if zero. Please use reasonable model and check input data.'
        )

    im_id = outs['im_id']

    infer_res = {}
    if 'bbox' in outs:
        if len(outs['bbox']) > 0 and len(outs['bbox'][0]) > 6:
            infer_res['bbox'] = get_det_poly_res(
                outs['bbox'], outs['bbox_num'], im_id, catid, bias=bias)
        else:
            infer_res['bbox'] = get_det_res(
                outs['bbox'], outs['bbox_num'], im_id, catid, bias=bias)

    if 'mask' in outs:
        # mask post process
        infer_res['mask'] = get_seg_res(outs['mask'], outs['bbox'],
                                        outs['bbox_num'], im_id, catid)

    if 'segm' in outs:
        infer_res['segm'] = get_solov2_segm_res(outs, im_id, catid)

    if 'keypoint' in outs:
        infer_res['keypoint'] = get_keypoint_res(outs, im_id)
        outs['bbox_num'] = [len(infer_res['keypoint'])]

    return infer_res

class Metric(paddle.metric.Metric):
    def name(self):
        return self.__class__.__name__

    def reset(self):
        pass

    def accumulate(self):
        pass

    # paddle.metric.Metric defined :metch:`update`, :meth:`accumulate`
    # :metch:`reset`, in ppdet, we also need following 2 methods:

    # abstract method for logging metric results
    def log(self):
        pass

    # abstract method for getting metric results
    def get_results(self):
        pass


class COCOMetric(Metric):
    def __init__(self, anno_file, **kwargs):
        self.anno_file = anno_file
        self.clsid2catid = kwargs.get('clsid2catid', None)
        if self.clsid2catid is None:
            self.clsid2catid, _ = get_categories('COCO', anno_file)
        self.classwise = kwargs.get('classwise', False)
        self.output_eval = kwargs.get('output_eval', None)
        # TODO: bias should be unified
        self.bias = kwargs.get('bias', 0)
        self.iou_type = kwargs.get('IouType', 'bbox')

        assert os.path.isfile(anno_file), \
                    "anno_file {} not a file".format(anno_file)

        if self.output_eval is not None:
            Path(self.output_eval).mkdir(exist_ok=True)

        self.reset()

    def reset(self):
        # only bbox and mask evaluation support currently
        self.results = {'bbox': [], 'mask': [], 'segm': [], 'keypoint': []}
        self.eval_results = {}

    def update(self, inputs, outputs):
        outs = {}
        # outputs Tensor -> numpy.ndarray
        for k, v in outputs.items():
            outs[k] = v.numpy() if isinstance(v, paddle.Tensor) else v

        # multi-scale inputs: all inputs have same im_id
        if isinstance(inputs, typing.Sequence):
            im_id = inputs[0]['im_id']
        else:
            im_id = inputs['im_id']
        outs['im_id'] = im_id.numpy() if isinstance(im_id,
                                                    paddle.Tensor) else im_id

        infer_results = get_infer_results(
            outs, self.clsid2catid, bias=self.bias)
        self.results['bbox'] += infer_results[
            'bbox'] if 'bbox' in infer_results else []
        self.results['mask'] += infer_results[
            'mask'] if 'mask' in infer_results else []
        self.results['segm'] += infer_results[
            'segm'] if 'segm' in infer_results else []
        self.results['keypoint'] += infer_results[
            'keypoint'] if 'keypoint' in infer_results else []

    def accumulate(self):
        if len(self.results['bbox']) > 0:
            output = "bbox.json"
            if self.output_eval:
                output = os.path.join(self.output_eval, output)
            with open(output, 'w') as f:
                json.dump(self.results['bbox'], f)
                logger.info('The bbox result is saved to bbox.json.')

        if len(self.results['mask']) > 0:
            output = "mask.json"
            if self.output_eval:
                output = os.path.join(self.output_eval, output)
            with open(output, 'w') as f:
                json.dump(self.results['mask'], f)
                logger.info('The mask result is saved to mask.json.')


        if len(self.results['segm']) > 0:
            output = "segm.json"
            if self.output_eval:
                output = os.path.join(self.output_eval, output)
            with open(output, 'w') as f:
                json.dump(self.results['segm'], f)
                logger.info('The segm result is saved to segm.json.')


        if len(self.results['keypoint']) > 0:
            output = "keypoint.json"
            if self.output_eval:
                output = os.path.join(self.output_eval, output)
            with open(output, 'w') as f:
                json.dump(self.results['keypoint'], f)
                logger.info('The keypoint result is saved to keypoint.json.')

    def log(self):
        pass

    def get_results(self):
        return self.eval_results



#   Copyright (c) 2020 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import numpy as np
from paddle.io import IterableDataset

class RecDataset(IterableDataset):
    def __init__(self, file_list):
        super(RecDataset, self).__init__()
        self.file_list = file_list
        self.init()

    def init(self):
        # Initialize the dataset
        from operator import mul
        padding = 0
        sparse_slots = "click 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26"
        self.sparse_slots = sparse_slots.strip().split(" ")
        self.dense_slots = ["dense_feature"]
        self.dense_slots_shape = [13]
        self.slots = self.sparse_slots + self.dense_slots
        self.slot2index = {}
        self.visit = {}
        for i in range(len(self.slots)):
            self.slot2index[self.slots[i]] = i
            self.visit[self.slots[i]] = False
        self.padding = padding
        count = 0
        for file in self.file_list:
            with open(file, "r") as rf:
                data = rf.readlines()
                count += len(data)
        self.nSamples = count

    def __len__(self):
        # Return the total number of samples in the dataset
        return self.nSamples

    def __iter__(self):
        # Iterate over the dataset
        full_lines = []
        self.data = []
        for file in self.file_list:
            with open(file, "r") as rf:
                for l in rf:
                    line = l.strip().split(" ")
                    output = [(i, []) for i in self.slots]
                    for i in line:
                        slot_feasign = i.split(":")
                        slot = slot_feasign[0]
                        if slot not in self.slots:
                            continue
                        if slot in self.sparse_slots:
                            feasign = int(slot_feasign[1])
                        else:
                            feasign = float(slot_feasign[1])
                        output[self.slot2index[slot]][1].append(feasign)
                        self.visit[slot] = True
                    for i in self.visit:
                        slot = i
                        if not self.visit[slot]:
                            if i in self.dense_slots:
                                output[self.slot2index[i]][1].extend(
                                    [self.padding] *
                                    self.dense_slots_shape[self.slot2index[i]])
                            else:
                                output[self.slot2index[i]][1].extend(
                                    [self.padding])
                        else:
                            self.visit[slot] = False
                    # sparse
                    output_list = []
                    for key, value in output[:-1]:
                        output_list.append(np.array(value).astype('int64'))
                    # dense
                    output_list.append(
                        np.array(output[-1][1]).astype("float32"))
                    # list
                    yield output_list


import numpy as np
import cv2
from typing import List
import os
import sys
import glob
import math
import magicmind.python.runtime as mm
from logger import Logger
# 实例化python logger类
log = Logger()

class CalibData(mm.CalibDataInterface):
    def __init__(self, args, shape: mm.Dims, max_samples: int, img_dir):
        super().__init__()
        self.task = args.task
        self.args = args
        if self.task == 'det':
            self.data_paths_ = glob.glob(img_dir + '/*.jpg')
        else :
            self.data_paths_ = glob.glob(img_dir + '/*.png')
        self.shape_ = shape
        self.batch_size = self.shape_.GetDimValue(0)
        self.max_samples_ = max_samples
        self.cur_sample_ = None
        self.cur_data_index_ = 0
 
    def get_shape(self):
        return self.shape_
 
    def get_data_type(self):
        return mm.DataType.FLOAT32
 
    def get_sample(self):
        return self.cur_sample_

    def preprocess_images(self,data_begin,data_end):
        imgs = []
        for i in range(data_begin,data_end):
            img_path = self.data_paths_[i]
            img = cv2.imread(img_path)
            if self.task == 'det':
                data = {'image': img}
                img, shape_list = db_preprocess(self.args.det_limit_side_len, self.args.det_limit_type, data)
            elif self.task =='rec':
                log.info("if you need rec int8 calib, please contact us")
                log.err("surpport use force_float16 or force_float32")
                # img = rec_preprocess()
            else:
                log.info("if you need cls int8 calib, please contact us")
                log.err("surpport use force_float16 or force_float32")
                # img = cls_preprocess()
            imgs.append(np.ascontiguousarray(img)[np.newaxis,:])
        return np.ascontiguousarray(np.concatenate(tuple(imgs), axis=0).astype(dtype = np.float32))
      
    def next(self):
        batch_size = self.batch_size
        data_begin = self.cur_data_index_
        data_end = data_begin + batch_size
        if data_end > self.max_samples_:
            return mm.Status(mm.Code.OUT_OF_RANGE, "Data end reached")
        self.cur_sample_ = self.preprocess_images(data_begin,data_end)
        self.cur_data_index_ = data_end
        return mm.Status.OK()
 
    def reset(self):
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        return mm.Status.OK()


def db_preprocess(det_limit_side_len, det_limit_type, data):
    std=[0.229, 0.224, 0.225]
    mean=[0.485, 0.456, 0.406]
    scale=1.0 / 255.0
    order='hwc'
    scale = np.float32(scale if scale is not None else 1.0 / 255.0)
    mean = mean if mean is not None else [0.485, 0.456, 0.406]
    std = std if std is not None else [0.229, 0.224, 0.225]
    shape = (3, 1, 1) if order == 'chw' else (1, 1, 3)
    mean = np.array(mean).reshape(shape).astype('float32')
    std = np.array(std).reshape(shape).astype('float32')
 
    img = data['image']
    src_h, src_w, _ = img.shape
    limit_side_len = det_limit_side_len
    h, w, c = img.shape
    # limit the max side
    if det_limit_type == 'max':
        if max(h,w) > limit_side_len:
            if h > w:
                ratio = float(limit_side_len)/h
            else:
                ratio = float(limit_side_len)/w
        else:
            ratio = 1
    elif det_limit_type == 'min':
        if min(h,w) < limit_side_len:
            if h < w:
                ratio = float(limit_side_len)/h
            else:
                ratio = float(limit_side_len)/w
        else:
            ratio = 1
    elif det_limit_type == 'resize_long':
            ratio = float(limit_side_len) / max(h, w)
    else:
        raise Exception('not support limit type, image ')
    resize_h = int(h * ratio)
    resize_w = int(w * ratio)

    resize_h = max(int(round(resize_h / 32) * 32), 32)
    resize_w = max(int(round(resize_w / 32) * 32), 32)

    try:
        if int(resize_w) <= 0 or int(resize_h) <= 0:
            return None, (None, None)
        img = cv2.resize(img, (int(resize_w), int(resize_h)))
    except:
        print(img.shape, resize_w, resize_h)
        sys.exit(0)

    ratio_h = resize_h / float(h)
    ratio_w = resize_w / float(w)  
    data['image'] = img
    data['shape'] = np.array([src_h, src_w, ratio_h, ratio_w])

    #nomalize image
    from PIL import Image
    if isinstance(img, Image.Image):
        img = np.array(img)
    assert isinstance(img,np.ndarray), "invalid input 'img' in NormalizeImage"
    data['image'] = (img.astype('float32') * scale - mean) / std

    #convert hwc image to chw image
    img = data['image']
    if isinstance(img, Image.Image):
        img = np.array(img)
    data['image'] = img.transpose((2, 0, 1))
    
    #Keep Keys
    keep_keys=['image', 'shape']
    data_list = []
    for key in keep_keys:
        data_list.append(data[key])
    return data_list

def rec_preprocess(rec_image_shape, limited_min_width, limited_max_width, beg_img_no,img_crop_list,indices,img_num, batch_num): 
    end_img_no = min(img_num, beg_img_no + batch_num)
    norm_img_batch = []

    #cambricon: ori cmd copied from predict_rec.py
    max_wh_ratio = 0
    for ino in range(beg_img_no, end_img_no):
        h, w = img_crop_list[indices[ino]].shape[0:2]
        wh_ratio = w * 1.0 / h
        max_wh_ratio = max(max_wh_ratio, wh_ratio)
    for ino in range(beg_img_no, end_img_no):
        norm_img = resize_norm_img_rec(rec_image_shape,limited_min_width, limited_max_width,img_crop_list[indices[ino]],max_wh_ratio)
        norm_img = norm_img[np.newaxis, :]
        norm_img_batch.append(norm_img)
    norm_img_batch = np.concatenate(norm_img_batch)
    norm_img_batch = norm_img_batch.copy()
    return norm_img_batch

def cls_preprocess(cls_image_shape, limited_min_width, limited_max_width, beg_img_no,img_crop_list,indices,img_num, batch_num):
    end_img_no = min(img_num, beg_img_no + batch_num)
    norm_img_batch = []
    max_wh_ratio = 0
    for ino in range(beg_img_no, end_img_no):
        h, w = img_crop_list[indices[ino]].shape[0:2]
        wh_ratio = w * 1.0 / h
        max_wh_ratio = max(max_wh_ratio, wh_ratio)
    for ino in range(beg_img_no, end_img_no):
        norm_img = resize_norm_img_cls(cls_image_shape,limited_min_width, limited_max_width,img_crop_list[indices[ino]],max_wh_ratio)
        norm_img = norm_img[np.newaxis, :]
        norm_img_batch.append(norm_img)
    norm_img_batch = np.concatenate(norm_img_batch)
    norm_img_batch = norm_img_batch.copy()
    return norm_img_batch

def resize_norm_img_cls(cls_image_shape,limited_min_width, limited_max_width, img, max_wh_ratio):
    cls_image_shape = [int(v) for v in cls_image_shape.split(",")]
    imgC, imgH, imgW = cls_image_shape
    h = img.shape[0]
    w = img.shape[1]
    ratio = w / float(h)
    imgW = max(min(imgW,limited_max_width),limited_min_width)
    ratio_imgH = math.ceil(imgH * ratio)
    ratio_imgH = max(ratio_imgH, limited_min_width)
    if ratio_imgH > imgW:
        resized_w = imgW
    else:
        resized_w = int(math.ceil(imgH * ratio))
    resized_image = cv2.resize(img, (resized_w, imgH))
    resized_image = resized_image.astype('float32')
    if cls_image_shape[0] == 1:
        resized_image = resized_image / 255
        resized_image = resized_image[np.newaxis, :]
    else:
        resized_image = resized_image.transpose((2,0,1)) / 255
    resized_image -= 0.5
    resized_image /= 0.5
    padding_im = np.zeros((imgC,imgH,imgW), dtype=np.float32)
    padding_im[:, :, 0:resized_w] = resized_image
    return padding_im

def resize_norm_img_rec(rec_image_shape,limited_min_width, limited_max_width, img, max_wh_ratio):
    rec_image_shape = [int(v) for v in rec_image_shape.split(",")]
    # 默认输入尺寸
    imgC, imgH, imgW = rec_image_shape
    assert imgC == img.shape[2]
    max_wh_ratio = max(max_wh_ratio, imgW / imgH)
    imgW = int((imgH * max_wh_ratio))
    imgW = max(min(imgW, limited_max_width),limited_min_width)
    h, w = img.shape[:2]
    ratio = w / float(h)
    #按比例缩放
    ratio_imgH = math.ceil(imgH * ratio)
    ratio_imgH = max(ratio_imgH, limited_min_width)
    if ratio_imgH > imgW:
        # 如大于默认宽度，则宽度为imgW
        resized_w = imgW
    else:
        #如小于默认宽度则以图片真实宽度为准
        resized_w = int(ratio_imgH)
    #缩放
    resized_image = cv2.resize(img, (resized_w, imgH))
    resized_image = resized_image.astype('float32')
    #归一化
    resized_image = resized_image.transpose((2, 0, 1)) / 255
    resized_image -= 0.5
    resized_image /= 0.5
    #对宽度不足的位置，补0
    padding_im = np.zeros((imgC, imgH, imgW), dtype=np.float32)
    padding_im[:, :, 0:resized_w] = resized_image
    return padding_im





#
# Copyright (c) 2021, NVIDIA CORPORATION. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
""" from https://github.com/keithito/tacotron """

import inflect
import re


_inflect = inflect.engine()
_comma_number_re = re.compile(r'([0-9][0-9\,]+[0-9])')
_decimal_number_re = re.compile(r'([0-9]+\.[0-9]+)')
_pounds_re = re.compile(r'£([0-9\,]*[0-9]+)')
_dollars_re = re.compile(r'\$([0-9\.\,]*[0-9]+)')
_ordinal_re = re.compile(r'[0-9]+(st|nd|rd|th)')
_number_re = re.compile(r'[0-9]+')


def _remove_commas(m):
  return m.group(1).replace(',', '')


def _expand_decimal_point(m):
  return m.group(1).replace('.', ' point ')


def _expand_dollars(m):
  match = m.group(1)
  parts = match.split('.')
  if len(parts) > 2:
    return match + ' dollars'  # Unexpected format
  dollars = int(parts[0]) if parts[0] else 0
  cents = int(parts[1]) if len(parts) > 1 and parts[1] else 0
  if dollars and cents:
    dollar_unit = 'dollar' if dollars == 1 else 'dollars'
    cent_unit = 'cent' if cents == 1 else 'cents'
    return '%s %s, %s %s' % (dollars, dollar_unit, cents, cent_unit)
  elif dollars:
    dollar_unit = 'dollar' if dollars == 1 else 'dollars'
    return '%s %s' % (dollars, dollar_unit)
  elif cents:
    cent_unit = 'cent' if cents == 1 else 'cents'
    return '%s %s' % (cents, cent_unit)
  else:
    return 'zero dollars'


def _expand_ordinal(m):
  return _inflect.number_to_words(m.group(0))


def _expand_number(m):
  num = int(m.group(0))
  if num > 1000 and num < 3000:
    if num == 2000:
      return 'two thousand'
    elif num > 2000 and num < 2010:
      return 'two thousand ' + _inflect.number_to_words(num % 100)
    elif num % 100 == 0:
      return _inflect.number_to_words(num // 100) + ' hundred'
    else:
      return _inflect.number_to_words(num, andword='', zero='oh', group=2).replace(', ', ' ')
  else:
    return _inflect.number_to_words(num, andword='')


def normalize_numbers(text):
  text = re.sub(_comma_number_re, _remove_commas, text)
  text = re.sub(_pounds_re, r'\1 pounds', text)
  text = re.sub(_dollars_re, _expand_dollars, text)
  text = re.sub(_decimal_number_re, _expand_decimal_point, text)
  text = re.sub(_ordinal_re, _expand_ordinal, text)
  text = re.sub(_number_re, _expand_number, text)
  return text



#    Copyright 2020 Division of Medical Image Computing, German Cancer Research Center (DKFZ), Heidelberg, Germany
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.


import argparse
from copy import deepcopy
from typing import Tuple, Union, List

import numpy as np
from batchgenerators.augmentations.utils import resize_segmentation
from nnunet.inference.segmentation_export import save_segmentation_nifti_from_softmax, save_segmentation_nifti
from batchgenerators.utilities.file_and_folder_operations import *
from multiprocessing import Process, Queue
import torch
import SimpleITK as sitk
import shutil
from multiprocessing import Pool
from nnunet.postprocessing.connected_components import load_remove_save, load_postprocessing
from nnunet.training.model_restore import load_model_and_checkpoint_files
from nnunet.training.network_training.nnUNetTrainer import nnUNetTrainer
from nnunet.utilities.one_hot_encoding import to_one_hot


def preprocess_save_to_queue(preprocess_fn, q, list_of_lists, output_files, segs_from_prev_stage, classes,
                             transpose_forward):
    # suppress output
    # sys.stdout = open(os.devnull, 'w')

    errors_in = []
    for i, l in enumerate(list_of_lists):
        try:
            output_file = output_files[i]
            print("preprocessing", output_file)
            d, _, dct = preprocess_fn(l)
            # print(output_file, dct)
            if segs_from_prev_stage[i] is not None:
                assert isfile(segs_from_prev_stage[i]) and segs_from_prev_stage[i].endswith(
                    ".nii.gz"), "segs_from_prev_stage" \
                                " must point to a " \
                                "segmentation file"
                seg_prev = sitk.GetArrayFromImage(sitk.ReadImage(segs_from_prev_stage[i]))
                # check to see if shapes match
                img = sitk.GetArrayFromImage(sitk.ReadImage(l[0]))
                assert all([i == j for i, j in zip(seg_prev.shape, img.shape)]), "image and segmentation from previous " \
                                                                                 "stage don't have the same pixel array " \
                                                                                 "shape! image: %s, seg_prev: %s" % \
                                                                                 (l[0], segs_from_prev_stage[i])
                seg_prev = seg_prev.transpose(transpose_forward)
                seg_reshaped = resize_segmentation(seg_prev, d.shape[1:], order=1)
                seg_reshaped = to_one_hot(seg_reshaped, classes)
                d = np.vstack((d, seg_reshaped)).astype(np.float32)
            """There is a problem with python process communication that prevents us from communicating objects 
            larger than 2 GB between processes (basically when the length of the pickle string that will be sent is 
            communicated by the multiprocessing.Pipe object then the placeholder (I think) does not allow for long 
            enough strings (lol). This could be fixed by changing i to l (for long) but that would require manually 
            patching system python code. We circumvent that problem here by saving softmax_pred to a npy file that will 
            then be read (and finally deleted) by the Process. save_segmentation_nifti_from_softmax can take either 
            filename or np.ndarray and will handle this automatically"""
            print(d.shape)
            if np.prod(d.shape) > (2e9 / 4 * 0.85):  # *0.85 just to be save, 4 because float32 is 4 bytes
                print(
                    "This output is too large for python process-process communication. "
                    "Saving output temporarily to disk")
                np.save(output_file[:-7] + ".npy", d)
                d = output_file[:-7] + ".npy"
            q.put((output_file, (d, dct)))
        except KeyboardInterrupt:
            raise KeyboardInterrupt
        except Exception as e:
            print("error in", l)
            print(e)
    q.put("end")
    if len(errors_in) > 0:
        print("There were some errors in the following cases:", errors_in)
        print("These cases were ignored.")
    else:
        print("This worker has ended successfully, no errors to report")
    # restore output
    # sys.stdout = sys.__stdout__


def preprocess_multithreaded(trainer, list_of_lists, output_files, num_processes=2, segs_from_prev_stage=None):
    if segs_from_prev_stage is None:
        segs_from_prev_stage = [None] * len(list_of_lists)

    num_processes = min(len(list_of_lists), num_processes)

    classes = list(range(1, trainer.num_classes))
    assert isinstance(trainer, nnUNetTrainer)
    q = Queue(1)
    processes = []
    for i in range(num_processes):
        pr = Process(target=preprocess_save_to_queue, args=(trainer.preprocess_patient, q,
                                                            list_of_lists[i::num_processes],
                                                            output_files[i::num_processes],
                                                            segs_from_prev_stage[i::num_processes],
                                                            classes, trainer.plans['transpose_forward']))
        pr.start()
        processes.append(pr)

    try:
        end_ctr = 0
        while end_ctr != num_processes:
            item = q.get()
            if item == "end":
                end_ctr += 1
                continue
            else:
                yield item

    finally:
        for p in processes:
            if p.is_alive():
                p.terminate()  # this should not happen but better safe than sorry right
            p.join()

        q.close()


def predict_cases(parameter_id, model, list_of_lists, output_filenames, folds, save_npz, num_threads_preprocessing,
                  num_threads_nifti_save, segs_from_prev_stage=None, do_tta=True, mixed_precision=True,
                  overwrite_existing=False,
                  all_in_gpu=False, step_size=0.5, checkpoint_name="model_final_checkpoint",
                  segmentation_export_kwargs: dict = None, disable_postprocessing: bool = False):
    """
    :param segmentation_export_kwargs:
    :param model: folder where the model is saved, must contain fold_x subfolders
    :param list_of_lists: [[case0_0000.nii.gz, case0_0001.nii.gz], [case1_0000.nii.gz, case1_0001.nii.gz], ...]
    :param output_filenames: [output_file_case0.nii.gz, output_file_case1.nii.gz, ...]
    :param folds: default: (0, 1, 2, 3, 4) (but can also be 'all' or a subset of the five folds, for example use (0, )
    for using only fold_0
    :param save_npz: default: False
    :param num_threads_preprocessing:
    :param num_threads_nifti_save:
    :param segs_from_prev_stage:
    :param do_tta: default: True, can be set to False for a 8x speedup at the cost of a reduced segmentation quality
    :param overwrite_existing: default: True
    :param mixed_precision: if None then we take no action. If True/False we overwrite what the model has in its init
    :return:
    """
    assert len(list_of_lists) == len(output_filenames)
    if segs_from_prev_stage is not None: assert len(segs_from_prev_stage) == len(output_filenames)

    pool = Pool(num_threads_nifti_save)
    results = []

    cleaned_output_files = []
    for o in output_filenames:
        dr, f = os.path.split(o)
        if len(dr) > 0:
            maybe_mkdir_p(dr)
        if not f.endswith(".nii.gz"):
            f, _ = os.path.splitext(f)
            f = f + ".nii.gz"
        cleaned_output_files.append(join(dr, f))

    if not overwrite_existing:
        print("number of cases:", len(list_of_lists))
        # if save_npz=True then we should also check for missing npz files
        not_done_idx = [i for i, j in enumerate(cleaned_output_files) if (not isfile(j)) or (save_npz and not isfile(j[:-7] + '.npz'))]

        cleaned_output_files = [cleaned_output_files[i] for i in not_done_idx]
        list_of_lists = [list_of_lists[i] for i in not_done_idx]
        if segs_from_prev_stage is not None:
            segs_from_prev_stage = [segs_from_prev_stage[i] for i in not_done_idx]

        print("number of cases that still need to be predicted:", len(cleaned_output_files))

    print("emptying cuda cache")
    torch.cuda.empty_cache()

    print("loading parameters for folds,", folds)
    trainer, params = load_model_and_checkpoint_files(model, folds, mixed_precision=mixed_precision,
                                                      checkpoint_name=checkpoint_name)

    if segmentation_export_kwargs is None:
        if 'segmentation_export_params' in trainer.plans.keys():
            force_separate_z = trainer.plans['segmentation_export_params']['force_separate_z']
            interpolation_order = trainer.plans['segmentation_export_params']['interpolation_order']
            interpolation_order_z = trainer.plans['segmentation_export_params']['interpolation_order_z']
        else:
            force_separate_z = None
            interpolation_order = 1
            interpolation_order_z = 0
    else:
        force_separate_z = segmentation_export_kwargs['force_separate_z']
        interpolation_order = segmentation_export_kwargs['interpolation_order']
        interpolation_order_z = segmentation_export_kwargs['interpolation_order_z']

    print("starting preprocessing generator")
    preprocessing = preprocess_multithreaded(trainer, [list_of_lists[0]], cleaned_output_files, 1,
                                             segs_from_prev_stage)
    print("starting prediction...")
    all_output_files = []
    for preprocessed in preprocessing:
        output_filename, (d, dct) = preprocessed
        all_output_files.append(all_output_files)
        if isinstance(d, str):
            data = np.load(d)
            os.remove(d)
            d = data

        trainer.load_checkpoint_ram(params[parameter_id], False)
        softmax = trainer.predict_preprocessed_data_return_seg_and_softmax(
            d, do_mirroring=do_tta, mirror_axes=trainer.data_aug_params['mirror_axes'], use_sliding_window=True,
            step_size=step_size, use_gaussian=True, all_in_gpu=all_in_gpu,
            mixed_precision=mixed_precision)[1]


def check_input_folder_and_return_caseIDs(input_folder, expected_num_modalities):
    print("This model expects %d input modalities for each image" % expected_num_modalities)
    files = subfiles(input_folder, suffix=".nii.gz", join=False, sort=True)

    maybe_case_ids = np.unique([i[:-12] for i in files])

    remaining = deepcopy(files)
    missing = []

    assert len(files) > 0, "input folder did not contain any images (expected to find .nii.gz file endings)"

    # now check if all required files are present and that no unexpected files are remaining
    for c in maybe_case_ids:
        for n in range(expected_num_modalities):
            expected_output_file = c + "_%04.0d.nii.gz" % n
            if not isfile(join(input_folder, expected_output_file)):
                missing.append(expected_output_file)
            else:
                remaining.remove(expected_output_file)

    print("Found %d unique case ids, here are some examples:" % len(maybe_case_ids),
          np.random.choice(maybe_case_ids, min(len(maybe_case_ids), 10)))
    print("If they don't look right, make sure to double check your filenames. They must end with _0000.nii.gz etc")

    if len(remaining) > 0:
        print("found %d unexpected remaining files in the folder. Here are some examples:" % len(remaining),
              np.random.choice(remaining, min(len(remaining), 10)))

    if len(missing) > 0:
        print("Some files are missing:")
        print(missing)
        raise RuntimeError("missing files in input_folder")

    return maybe_case_ids


def predict_from_folder(parameter_id: int, model: str, input_folder: str, output_folder: str, folds: Union[Tuple[int], List[int]],
                        save_npz: bool, num_threads_preprocessing: int, num_threads_nifti_save: int,
                        lowres_segmentations: Union[str, None],
                        part_id: int, num_parts: int, tta: bool, mixed_precision: bool = True,
                        overwrite_existing: bool = True, mode: str = 'normal', overwrite_all_in_gpu: bool = None,
                        step_size: float = 0.5, checkpoint_name: str = "model_final_checkpoint",
                        segmentation_export_kwargs: dict = None, disable_postprocessing: bool = False):
    """
        here we use the standard naming scheme to generate list_of_lists and output_files needed by predict_cases

    :param model:
    :param input_folder:
    :param output_folder:
    :param folds:
    :param save_npz:
    :param num_threads_preprocessing:
    :param num_threads_nifti_save:
    :param lowres_segmentations:
    :param part_id:
    :param num_parts:
    :param tta:
    :param mixed_precision:
    :param overwrite_existing: if not None then it will be overwritten with whatever is in there. None is default (no overwrite)
    :return:
    """
    maybe_mkdir_p(output_folder)
    shutil.copy(join(model, 'plans.pkl'), output_folder)

    assert isfile(join(model, "plans.pkl")), "Folder with saved model weights must contain a plans.pkl file"
    expected_num_modalities = load_pickle(join(model, "plans.pkl"))['num_modalities']

    # check input folder integrity
    case_ids = check_input_folder_and_return_caseIDs(input_folder, expected_num_modalities)

    output_files = [join(output_folder, i + ".nii.gz") for i in case_ids]
    all_files = subfiles(input_folder, suffix=".nii.gz", join=False, sort=True)
    list_of_lists = [[join(input_folder, i) for i in all_files if i[:len(j)].startswith(j) and
                      len(i) == (len(j) + 12)] for j in case_ids]

    if lowres_segmentations is not None:
        assert isdir(lowres_segmentations), "if lowres_segmentations is not None then it must point to a directory"
        lowres_segmentations = [join(lowres_segmentations, i + ".nii.gz") for i in case_ids]
        assert all([isfile(i) for i in lowres_segmentations]), "not all lowres_segmentations files are present. " \
                                                               "(I was searching for case_id.nii.gz in that folder)"
        lowres_segmentations = lowres_segmentations[part_id::num_parts]
    else:
        lowres_segmentations = None

    all_in_gpu = False
    predict_cases(parameter_id, model, list_of_lists[part_id::num_parts], output_files[part_id::num_parts], folds,
                             save_npz, num_threads_preprocessing, num_threads_nifti_save, lowres_segmentations, tta,
                             mixed_precision=mixed_precision, overwrite_existing=overwrite_existing,
                             all_in_gpu=all_in_gpu,
                             step_size=step_size, checkpoint_name=checkpoint_name,
                             segmentation_export_kwargs=segmentation_export_kwargs,
                             disable_postprocessing=disable_postprocessing)



def main(argv=None):
    parser = argparse.ArgumentParser()
    parser.add_argument("-i", '--input_folder', help="Must contain all modalities for each patient in the correct"
                                                     " order (same as training). Files must be named "
                                                     "CASENAME_XXXX.nii.gz where XXXX is the modality "
                                                     "identifier (0000, 0001, etc)", required=True)
    parser.add_argument('-o', "--output_folder", default='saved_pt', help="folder for saving predictions")
    parser.add_argument('-m', '--model_output_folder',
                        help='model output folder. Will automatically discover the folds '
                             'that were '
                             'run and use those as an ensemble', required=True)
    parser.add_argument('-f', '--folds', nargs='+', default='None', help="folds to use for prediction. Default is None "
                                                                         "which means that folds will be detected "
                                                                         "automatically in the model output folder")
    parser.add_argument('-z', '--save_npz', required=False, action='store_true', help="use this if you want to ensemble"
                                                                                      " these predictions with those of"
                                                                                      " other models. Softmax "
                                                                                      "probabilities will be saved as "
                                                                                      "compresed numpy arrays in "
                                                                                      "output_folder and can be merged "
                                                                                      "between output_folders with "
                                                                                      "merge_predictions.py")
    parser.add_argument('-l', '--lowres_segmentations', required=False, default='None', help="if model is the highres "
                                                                                             "stage of the cascade then you need to use -l to specify where the segmentations of the "
                                                                                             "corresponding lowres unet are. Here they are required to do a prediction")
    parser.add_argument("--part_id", type=int, required=False, default=0, help="Used to parallelize the prediction of "
                                                                               "the folder over several GPUs. If you "
                                                                               "want to use n GPUs to predict this "
                                                                               "folder you need to run this command "
                                                                               "n times with --part_id=0, ... n-1 and "
                                                                               "--num_parts=n (each with a different "
                                                                               "GPU (for example via "
                                                                               "CUDA_VISIBLE_DEVICES=X)")
    parser.add_argument("--num_parts", type=int, required=False, default=1,
                        help="Used to parallelize the prediction of "
                             "the folder over several GPUs. If you "
                             "want to use n GPUs to predict this "
                             "folder you need to run this command "
                             "n times with --part_id=0, ... n-1 and "
                             "--num_parts=n (each with a different "
                             "GPU (via "
                             "CUDA_VISIBLE_DEVICES=X)")
    parser.add_argument("--num_threads_preprocessing", required=False, default=6, type=int, help=
    "Determines many background processes will be used for data preprocessing. Reduce this if you "
    "run into out of memory (RAM) problems. Default: 6")
    parser.add_argument("--num_threads_nifti_save", required=False, default=2, type=int, help=
    "Determines many background processes will be used for segmentation export. Reduce this if you "
    "run into out of memory (RAM) problems. Default: 2")
    parser.add_argument("--tta", required=False, type=int, default=1, help="Set to 0 to disable test time data "
                                                                           "augmentation (speedup of factor "
                                                                           "4(2D)/8(3D)), "
                                                                           "lower quality segmentations")
    parser.add_argument("--overwrite_existing", required=False, type=int, default=1, help="Set this to 0 if you need "
                                                                                          "to resume a previous "
                                                                                          "prediction. Default: 1 "
                                                                                          "(=existing segmentations "
                                                                                          "in output_folder will be "
                                                                                          "overwritten)")
    parser.add_argument("--mode", type=str, default="normal", required=False)
    parser.add_argument("--parameter_id", type=int, default=0, required=True)

    parser.add_argument("--all_in_gpu", type=str, default="None", required=False, help="can be None, False or True")
    parser.add_argument("--step_size", type=float, default=0.5, required=False, help="don't touch")
    # parser.add_argument("--interp_order", required=False, default=3, type=int,
    #                     help="order of interpolation for segmentations, has no effect if mode=fastest")
    # parser.add_argument("--interp_order_z", required=False, default=0, type=int,
    #                     help="order of interpolation along z is z is done differently")
    # parser.add_argument("--force_separate_z", required=False, default="None", type=str,
    #                     help="force_separate_z resampling. Can be None, True or False, has no effect if mode=fastest")
    parser.add_argument('--disable_mixed_precision', default=False, action='store_true', required=False,
                        help='Predictions are done with mixed precision by default. This improves speed and reduces '
                             'the required vram. If you want to disable mixed precision you can set this flag. Note '
                             'that yhis is not recommended (mixed precision is ~2x faster!)')

    args = parser.parse_args()
    input_folder = args.input_folder
    part_id = args.part_id
    parameter_id = args.parameter_id 
    num_parts = args.num_parts
    model = args.model_output_folder
    folds = args.folds
    save_npz = args.save_npz
    lowres_segmentations = args.lowres_segmentations
    num_threads_preprocessing = args.num_threads_preprocessing
    num_threads_nifti_save = args.num_threads_nifti_save
    tta = args.tta
    step_size = args.step_size
    overwrite = args.overwrite_existing
    mode = args.mode
    all_in_gpu = args.all_in_gpu

    if lowres_segmentations == "None":
        lowres_segmentations = None

    if isinstance(folds, list):
        if folds[0] == 'all' and len(folds) == 1:
            pass
        else:
            folds = [int(i) for i in folds]
    elif folds == "None":
        folds = None
    else:
        raise ValueError("Unexpected value for argument folds")

    if tta == 0:
        tta = False
    elif tta == 1:
        tta = True
    else:
        raise ValueError("Unexpected value for tta, Use 1 or 0")

    if overwrite == 0:
        overwrite = False
    elif overwrite == 1:
        overwrite = True
    else:
        raise ValueError("Unexpected value for overwrite, Use 1 or 0")

    assert all_in_gpu in ['None', 'False', 'True']
    if all_in_gpu == "None":
        all_in_gpu = None
    elif all_in_gpu == "True":
        all_in_gpu = True
    elif all_in_gpu == "False":
        all_in_gpu = False

    predict_from_folder(parameter_id, model, input_folder, '/tmp', folds, save_npz, num_threads_preprocessing,
                        num_threads_nifti_save, lowres_segmentations, part_id, num_parts, tta,
                        mixed_precision=not args.disable_mixed_precision,
                        overwrite_existing=overwrite, mode=mode, overwrite_all_in_gpu=all_in_gpu, step_size=step_size)

    import shutil
    shutil.move('2dunet_tmp.pt', '%s/2dunet_%i.pt' % (args.output_folder, parameter_id))
    shutil.move('calib_data.pt', '%s/calib_data_%i.pt' % (args.output_folder, parameter_id))

if __name__ == "__main__":
    main()




import numpy as np
from magicmind.python.common.types import get_numpy_dtype_by_datatype
from typing import List
import magicmind.python.runtime as mm
from torchvision import transforms, datasets
import os
import glob
import math
import torch
from mmcv import Config, DictAction
from mmaction.datasets import build_dataloader, build_dataset

def build_data_loader(config_file):
    cfg = Config.fromfile(config_file)
    cfg.merge_from_dict({})
    cfg.data.test.test_mode = True
    cfg.setdefault('module_hooks', [])
    # build the dataloader
    dataset = build_dataset(cfg.data.test, dict(test_mode=True))
    dataloader_setting = dict(
        videos_per_gpu=cfg.data.get('videos_per_gpu', 1),
        workers_per_gpu=cfg.data.get('workers_per_gpu', 1),
        dist=False,
        shuffle=False)
    dataloader_setting = dict(dataloader_setting,
                              **cfg.data.get('test_dataloader', {}))
    data_loader = build_dataloader(dataset, **dataloader_setting)
    return data_loader


def preprocess(batch_size,config_file):
    loaders = build_data_loader(config_file)
    for data in loaders:
        np_inputs = data['imgs'].cpu().numpy()
        break
    return np_inputs
    

class CalibData(mm.CalibDataInterface):
    def __init__(self, shape: mm.Dims, max_samples: int,config_file:str):
        super().__init__()
        self.shape_ = shape
        self.max_samples_ = max_samples
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        self.config_file = config_file
 
    def get_shape(self):
        return self.shape_
 
    def get_data_type(self):
        return mm.DataType.FLOAT32
 
    def get_sample(self):
        return self.cur_sample_
     
    def preprocess_images(self, data_begin: int, data_end: int) -> np.ndarray:
        after_preprocess = preprocess(self.max_samples_,self.config_file)
        return np.ascontiguousarray(after_preprocess)
 
    def next(self):
        batch_size = self.shape_.GetDimValue(0)
        data_begin = self.cur_data_index_
        data_end = data_begin + batch_size
        if data_end > self.max_samples_:
            return mm.Status(mm.Code.OUT_OF_RANGE, "Data end reached")
        self.cur_sample_ = self.preprocess_images(data_begin, data_end)
        self.cur_data_index_ = data_end
        return mm.Status.OK()
 
    def reset(self):
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        return mm.Status.OK()


import magicmind.python.runtime as mm
import argparse
import numpy as np
import cv2
import torch
import math
import sys
import os

sys.path.append("..")
from gen_model.calibrator import letterbox, coco_dataset
from utils import Record

from mm_runner import MMRunner
from logger import Logger

log = Logger()

parser = argparse.ArgumentParser()
parser.add_argument("--device_id", "--device_id", type=int, default=0, help="device_id")
parser.add_argument(
    "--magicmind_model",
    "--magicmind_model",
    type=str,
    default="../data/models/yolov5_pytorch_model_qint8_mixed_float16_true_1",
)
parser.add_argument(
    "--image_dir",
    "--image_dir",
    type=str,
    default="../../../../datasets/coco/val2017",
    help="coco val datasets",
)
parser.add_argument(
    "--image_num", "--image_num", type=int, default=10, help="image number"
)
parser.add_argument(
    "--file_list",
    "--file_list",
    type=str,
    default="coco_file_list_5000.txt",
    help="coco file list",
)
parser.add_argument("--label_path", "--label_path", type=str, default="coco.names")
parser.add_argument(
    "--output_dir", "--output_dir", type=str, default="../data/images/output"
)
parser.add_argument(
    "--imgsz", "--imgsz", default=640, type=int, help="inference size (pixels)"
)
parser.add_argument(
    "--batch_size", "--batch_size", default=8, type=int, help="inference size (pixels)"
)
parser.add_argument("--save_img", "--save_img", type=bool, default=False)

if __name__ == "__main__":
    args = parser.parse_args()
    if not os.path.exists(args.magicmind_model):
        log.error(args.magicmind_model + " does not exist.")
        exit()

    # model 定义
    model = MMRunner(mm_file=args.magicmind_model, device_id=args.device_id)

    batch_size = args.batch_size
    image_num = args.image_num
    imgsz = args.imgsz

    img_names = [None for i in range(batch_size)]
    imgs = np.empty([batch_size, imgsz, imgsz, 3], dtype=np.uint8)
    show_imgs = [
        None for i in range(batch_size)
    ]  # np.empty([batch_size,imgsz,imgsz,3],dtype=np.uint8)
    ratios = np.empty([batch_size])

    if isinstance(args.imgsz, int):
        img_size = (imgsz, imgsz)
        dataset = coco_dataset(
            file_list_txt=args.file_list, image_dir=args.image_dir, count=args.image_num
        )
        rem_img_num = image_num % batch_size
        img_idx = 0
        batch_counter = 0
        log.info("Start run ...")
        from tqdm import tqdm

        for img, img_path in tqdm(dataset, total=args.image_num):
            infer_batch_size = (
                batch_size if img_idx < (image_num - rem_img_num) else rem_img_num
            )
            img_names[batch_counter % infer_batch_size] = os.path.splitext(
                img_path.split("/")[-1]
            )[0]

            # pre-process
            show_imgs[batch_counter % infer_batch_size] = img
            img, ratio = letterbox(img, img_size)
            img = img[:, :, ::-1]  # BGR to RGB
            imgs[batch_counter % infer_batch_size, :, :, :] = img
            ratios[batch_counter % infer_batch_size] = ratio

            batch_counter += 1
            img_idx += 1

            if batch_counter % infer_batch_size == 0:
                batch_counter = 0
                inputs = [imgs]

                # inference
                outputs = model(inputs)

                # post-process
                preds = torch.from_numpy(outputs[0])
                detection_nums = torch.from_numpy(outputs[1])
                for pred_idx in range(infer_batch_size):
                    show_img = np.array(show_imgs[pred_idx])
                    pred = preds[pred_idx]
                    detection_num = detection_nums[pred_idx]
                    reshape_value = torch.reshape(pred, (-1, 1))
                    src_h, src_w = show_img.shape[0], show_img.shape[1]
                    scale_w = ratios[pred_idx] * src_w
                    scale_h = ratios[pred_idx] * src_h
                    record = Record(
                        args.output_dir + "/" + img_names[pred_idx] + ".txt"
                    )
                    name_dict = np.loadtxt(args.label_path, dtype="str", delimiter="\n")
                    for k in range(detection_num):
                        class_id = int(reshape_value[k * 7 + 1])
                        score = float(reshape_value[k * 7 + 2])
                        xmin = max(0, min(reshape_value[k * 7 + 3], img_size[1]))
                        xmax = max(0, min(reshape_value[k * 7 + 5], img_size[1]))
                        ymin = max(0, min(reshape_value[k * 7 + 4], img_size[0]))
                        ymax = max(0, min(reshape_value[k * 7 + 6], img_size[0]))
                        xmin = (xmin - (img_size[1] - scale_w) / 2) / ratios[pred_idx]
                        xmax = (xmax - (img_size[1] - scale_w) / 2) / ratios[pred_idx]
                        ymin = (ymin - (img_size[0] - scale_h) / 2) / ratios[pred_idx]
                        ymax = (ymax - (img_size[0] - scale_h) / 2) / ratios[pred_idx]
                        xmin = float(max(0, xmin))
                        xmax = float(max(0, xmax))
                        ymin = float(max(0, ymin))
                        ymax = float(max(0, ymax))
                        result = (
                            name_dict[class_id]
                            + ","
                            + str(score)
                            + ","
                            + str(xmin)
                            + ","
                            + str(ymin)
                            + ","
                            + str(xmax)
                            + ","
                            + str(ymax)
                        )
                        record.write(result, False)
                        if args.save_img:
                            cv2.rectangle(
                                show_img,
                                (int(xmin), int(ymin)),
                                (int(xmax), int(ymax)),
                                (0, 255, 0),
                            )
                            text = name_dict[class_id] + ": " + str(score)
                            text_size, _ = cv2.getTextSize(text, 0, 0.5, 1)
                            cv2.putText(
                                show_img,
                                text,
                                (int(xmin), int(ymin) + text_size[1]),
                                0,
                                0.5,
                                (255, 255, 255),
                                1,
                            )
                    if args.save_img:
                        cv2.imwrite(
                            args.output_dir + "/" + img_names[pred_idx] + ".jpg",
                            show_img,
                        )



import argparse
import magicmind.python.runtime as mm
import torch
import os
from tqdm import tqdm
import transformers
from transformers.data.metrics.squad_metrics import (
    compute_predictions_logits,
    squad_evaluate,
)
import warnings

warnings.filterwarnings("ignore")
from preprocess import preprocess
from postprocess import postprocess
import sys
import numpy as np

from utils import Record
import pickle

from mm_runner import MMRunner
from logger import Logger

log = Logger()

parser = argparse.ArgumentParser()
parser.add_argument("--device_id", "--device_id", type=int, default=0, help="device_id")
parser.add_argument(
    "--magicmind_model",
    "--magicmind_model",
    type=str,
    default="../data/models/bert_squad_pytorch_model",
)
parser.add_argument(
    "--json_file",
    "--json_file",
    type=str,
    default="../../../../../../datasets/squadv1.1/dev-v1.1.json",
)
parser.add_argument(
    "--batch_size", "--batch_size", type=int, default=16, help="batch_size"
)
parser.add_argument(
    "--max_seq_length", "--max_seq_length", type=int, default=384, help="max_seq_length"
)
parser.add_argument("--compute_accuracy", "--compute_accuracy", type=bool, default=True)
parser.add_argument(
    "--output_dir", "--output_dir", type=str, default="../data/jsons/output"
)
parser.add_argument(
    "--acc_result",
    "--acc_result",
    type=str,
    default="../data/jsons/output/acc_result.txt",
)

if __name__ == "__main__":
    args = parser.parse_args()
    if not os.path.exists(args.magicmind_model):
        log.info("please generate magicmind model first!!!")
        exit()

    # model 定义
    model = MMRunner(mm_file=args.magicmind_model, device_id=args.device_id)

    tokenizer, examples, features, eval_dataloader = preprocess(
        args.json_file, 1, args.max_seq_length
    )

    data_num = len(eval_dataloader)
    batch_size = args.batch_size
    max_seq_length = args.max_seq_length
    rem_data_num = data_num % args.batch_size
    data_idx = 0
    batch_counter = 0
    batch_input_1 = np.empty([batch_size, max_seq_length])
    batch_input_2 = np.empty([batch_size, max_seq_length])
    batch_input_3 = np.empty([batch_size, max_seq_length])
    post_batch = np.empty([batch_size, 1], dtype=np.int)
    all_results = []

    for batch in tqdm(eval_dataloader, desc="Evaluating"):
        infer_batch_size = (
            batch_size if data_idx < (data_num - rem_data_num) else rem_data_num
        )

        batch = tuple(t for t in batch)

        batch_input_1[batch_counter % infer_batch_size] = batch[0][0].numpy()
        batch_input_2[batch_counter % infer_batch_size] = batch[1][0].numpy()
        batch_input_3[batch_counter % infer_batch_size] = batch[2][0].numpy()
        post_batch[batch_counter % infer_batch_size] = batch[3][0].numpy()

        batch_counter += 1
        data_idx += 1

        if batch_counter % infer_batch_size == 0:
            batch_counter = 0
            inputs = [batch_input_1, batch_input_2, batch_input_3]

            # infer
            outputs = model(inputs)

            post_outputs = [
                outputs[0][0:infer_batch_size, :],
                outputs[1][0:infer_batch_size, :],
            ]

            # process output
            all_results.extend(
                postprocess(
                    features,
                    torch.from_numpy(post_batch[0:infer_batch_size, :]),
                    post_outputs,
                )
            )

    ### 精度计算
    if args.compute_accuracy:
        acc_result = Record(args.acc_result)
        output_prediction_file = os.path.join(args.output_dir, "predictions.json")
        output_nbest_file = os.path.join(args.output_dir, "nbest_predictions.json")
        predictions = compute_predictions_logits(
            examples,
            features,
            all_results,
            20,  # n best size
            30,  # max answer length
            False,  # do lower case
            output_prediction_file,
            output_nbest_file,
            None,
            False,
            False,
            0.0,
            tokenizer,
        )
        squad_acc = squad_evaluate(examples, predictions)
        acc_result.write("SQUAD results: {}".format(squad_acc), True)



# from __future__ import print_function
import argparse
import os,sys
import numpy as np
import magicmind.python.runtime as mm
import cv2

_classes = ('__background__',
        'aeroplane', 'bicycle', 'bird', 'boat',
        'bottle', 'bus', 'car', 'cat', 'chair',
        'cow', 'diningtable', 'dog', 'horse',
        'motorbike', 'person', 'pottedplant',
        'sheep', 'sofa', 'train', 'tvmonitor')

def parse_args():
    """Parse input arguments."""
    parser = argparse.ArgumentParser(description='mAP Calculation')
    parser.add_argument('--magicmind_model', type=str, required=True)
    parser.add_argument('--result_path', help='The result data path', type=str)
    parser.add_argument('--devkit_path', help='VOCdevkit path', type=str)
    parser.add_argument('--save_img', type=bool, default=False)
    args = parser.parse_args()

    return args

def get_results(path, pred_dir, save_img):
    model = mm.Model()
    model.deserialize_from_file(args.magicmind_model)
    threshold = 0.01
    input_size = [300, 300]

    def preprocess_images(image_path: str, dst_size: list):
        imgs = []
        dst_h, dst_w = dst_size[0], dst_size[1]
        ori = cv2.imread(image_path)
        ori_shape = ori.shape
        scale = [float(ori_shape[0]) / float(dst_h), float(ori_shape[1]) / float(dst_w)]
        img = cv2.resize(ori, (dst_w, dst_h), interpolation=cv2.INTER_LINEAR)
        # mean std
        img = img.astype(np.float32)
        img -= 127.5
        img *= 0.007843
        imgs.append(np.ascontiguousarray(img)[np.newaxis,:])
        # batch
        return np.ascontiguousarray(np.concatenate(tuple(imgs), axis=0)), ori, scale

    # 读取测试图片文件名
    filenames = []
    with open(os.path.join(path,'VOC2007/ImageSets/Main/test.txt'), 'r') as f:
        filenames = f.readlines()
    # 生成图片列表文件
    images_path = [os.path.join(path, 'VOC2007/JPEGImages/' + filename.strip() + '.jpg')\
              for filename in filenames]
    #创建保存结果的文件夹

    voc_preds_files = []
    folder = os.path.exists(pred_dir)
    if not folder: 
        os.makedirs(pred_dir)
    for t in _classes:
        voc_preds_files.append(open(pred_dir + '/comp3_det_test_' + t + '.txt', 'w'))
    total_num = images_path.__len__()
    print('total image number:{}'.format(total_num))
    with mm.System():
        dev = mm.Device()
        dev.id = 0
        econfig = mm.Model.EngineConfig()
        econfig.device_type = "MLU"
        engine = model.create_i_engine(econfig)
        assert engine != None, "Failed to create engine"
        context = engine.create_i_context()
        queue = dev.create_queue()
        assert queue != None

        inputs = context.create_inputs()
        for i, path in enumerate(images_path):
            print('{}/{}:{}'.format(i+1, total_num, path))
            img, ori, scale = preprocess_images(path, input_size)
            inputs[0].from_numpy(img)
            inputs[0].to(dev)
            outputs = []
            assert context.enqueue(inputs, outputs, queue).ok()
            assert queue.sync().ok()

            results = outputs[0].asnumpy()
            out_shape = results.shape
            bbox_size = 7  # every 7 values form a bounding box.
            assert out_shape[1] == bbox_size

            bbox_num = out_shape[0]
            for bbox in results:
                batch_id = bbox[0]
                category = int(bbox[1])
                # background
                if 0 == category: continue
                
                if (threshold > bbox[2]): continue

                left = float(bbox[3] * float(img.shape[2]) * scale[1])  # left
                top = float(bbox[4] * float(img.shape[1]) * scale[0])  # top
                right = float(bbox[5] * float(img.shape[2]) * scale[1])  # right
                bottom = float(bbox[6] * float(img.shape[1]) * scale[0])  # bottom
                
                if left >= right or top >= bottom : continue
                # check border
                left = max(0, left)
                right = min(right, ori.shape[1])
                top = max(top, 0)
                bottom = min(bottom, ori.shape[0])

                # write filename first
                voc_preds_files[category].write(filenames[i].strip())
                # voc_preds_files[category].write(' ' + str(category))
                voc_preds_files[category].write(' ' + str(bbox[2]))
                voc_preds_files[category].write(' ' + str(left))
                voc_preds_files[category].write(' ' + str(top))
                voc_preds_files[category].write(' ' + str(right))
                voc_preds_files[category].write(' ' + str(bottom))
                voc_preds_files[category].write('\n')

                if save_img:
                    cv2.rectangle(ori, (int(left), int(top)), (int(right), int(bottom)), (255, 255, 255))
                    text = _classes[category] + ": " + str(bbox[2])
                    text_size, _ = cv2.getTextSize(text, 0, 0.5, 1)
                    cv2.putText(ori, text, (int(left), int(top) + text_size[1]), 0, 0.5, (255, 255, 255), 1)
            if save_img:
                cv2.imwrite(os.path.join(pred_dir, filenames[i].strip() + '.jpg'), ori)

    for file in voc_preds_files:
        file.close()


if __name__ == '__main__':
    args = parse_args()
    get_results(args.devkit_path, args.result_path, args.save_img)



import numpy as np
import magicmind.python.runtime as mm
import cv2
import os
import glob
import math
import logging
from PIL import Image
from torchvision.transforms import transforms
import copy
import functools
import torch
from torch.utils.data.dataloader import default_collate
from PIL import Image
import json
from pathlib import Path
import torch
import torch.utils.data as data
from logger import Logger

log = Logger()


def get_class_labels(data):
    class_labels_map = {}
    index = 0
    for class_label in data["labels"]:
        class_labels_map[class_label] = index
        index += 1
    return class_labels_map


def get_database(data, subset, root_path, video_path_formatter):
    video_ids = []
    video_paths = []
    annotations = []

    for key, value in data["database"].items():
        this_subset = value["subset"]
        if this_subset == subset:
            video_ids.append(key)
            annotations.append(value["annotations"])
            if "video_path" in value:
                video_paths.append(Path(value["video_path"]))
            else:
                label = value["annotations"]["label"]
                video_paths.append(video_path_formatter(root_path, label, key))

    return video_ids, video_paths, annotations


class VideoDataset(data.Dataset):
    def __init__(
        self,
        root_path,
        annotation_path,
        subset,
        spatial_transform=None,
        temporal_transform=None,
        target_transform=None,
        video_loader=None,
        video_path_formatter=(
            lambda root_path, label, video_id: root_path / label / video_id
        ),
        image_name_formatter=lambda x: f"image_{x:05d}.jpg",
        target_type="label",
    ):
        self.data, self.class_names = self.__make_dataset(
            root_path, annotation_path, subset, video_path_formatter
        )

        self.spatial_transform = spatial_transform
        self.temporal_transform = temporal_transform
        self.target_transform = target_transform

        self.loader = video_loader

        self.target_type = target_type

    def __make_dataset(self, root_path, annotation_path, subset, video_path_formatter):
        with annotation_path.open("r") as f:
            data = json.load(f)
        video_ids, video_paths, annotations = get_database(
            data, subset, root_path, video_path_formatter
        )
        class_to_idx = get_class_labels(data)
        idx_to_class = {}
        for name, label in class_to_idx.items():
            idx_to_class[label] = name

        n_videos = len(video_ids)
        dataset = []
        for i in range(n_videos):
            if i % (n_videos // 5) == 0:
                print("dataset loading [{}/{}]".format(i, len(video_ids)))

            if "label" in annotations[i]:
                label = annotations[i]["label"]
                label_id = class_to_idx[label]
            else:
                label = "test"
                label_id = -1

            video_path = video_paths[i]
            if not video_path.exists():
                continue

            segment = annotations[i]["segment"]
            if segment[1] == 1:
                continue

            frame_indices = list(range(segment[0], segment[1]))
            sample = {
                "video": video_path,
                "segment": segment,
                "frame_indices": frame_indices,
                "video_id": video_ids[i],
                "label": label_id,
            }
            dataset.append(sample)
        return dataset, idx_to_class

    def __loading(self, path, frame_indices):
        clip = self.loader(path, frame_indices)
        if self.spatial_transform is not None:
            self.spatial_transform.randomize_parameters()
            clip = [self.spatial_transform(img) for img in clip]
        clip = torch.stack(clip, 0).permute(1, 0, 2, 3)

        return clip

    def __getitem__(self, index):
        path = self.data[index]["video"]
        if isinstance(self.target_type, list):
            target = [self.data[index][t] for t in self.target_type]
        else:
            target = self.data[index][self.target_type]

        frame_indices = self.data[index]["frame_indices"]
        if self.temporal_transform is not None:
            frame_indices = self.temporal_transform(frame_indices)

        clip = self.__loading(path, frame_indices)

        if self.target_transform is not None:
            target = self.target_transform(target)

        return clip, target

    def __len__(self):
        return len(self.data)


def collate_fn(batch):
    batch_clips, batch_targets = zip(*batch)

    batch_clips = [clip for multi_clips in batch_clips for clip in multi_clips]
    batch_targets = [
        target for multi_targets in batch_targets for target in multi_targets
    ]

    target_element = batch_targets[0]
    if isinstance(target_element, int) or isinstance(target_element, str):
        return default_collate(batch_clips), default_collate(batch_targets)
    else:
        return default_collate(batch_clips), batch_targets


class VideoDatasetMultiClips(VideoDataset):
    def __loading(self, path, video_frame_indices):
        clips = []
        segments = []
        for clip_frame_indices in video_frame_indices:
            clip = self.loader(path, clip_frame_indices)
            if self.spatial_transform is not None:
                self.spatial_transform.randomize_parameters()
                clip = [self.spatial_transform(img) for img in clip]
            clips.append(torch.stack(clip, 0).permute(1, 0, 2, 3))
            segments.append([min(clip_frame_indices), max(clip_frame_indices) + 1])

        return clips, segments

    def __getitem__(self, index):
        path = self.data[index]["video"]

        video_frame_indices = self.data[index]["frame_indices"]
        if self.temporal_transform is not None:
            video_frame_indices = self.temporal_transform(video_frame_indices)

        clips, segments = self.__loading(path, video_frame_indices)

        if isinstance(self.target_type, list):
            target = [self.data[index][t] for t in self.target_type]
        else:
            target = self.data[index][self.target_type]

        if "segment" in self.target_type:
            if isinstance(self.target_type, list):
                segment_index = self.target_type.index("segment")
                targets = []
                for s in segments:
                    targets.append(copy.deepcopy(target))
                    targets[-1][segment_index] = s
            else:
                targets = segments
        else:
            targets = [target for _ in range(len(segments))]
        return clips, targets


class Compose(transforms.Compose):
    def randomize_parameters(self):
        for t in self.transforms:
            t.randomize_parameters()


class ToTensor(transforms.ToTensor):
    def randomize_parameters(self):
        pass


class Normalize(transforms.Normalize):
    def randomize_parameters(self):
        pass


class ScaleValue(object):
    def __init__(self, s):
        self.s = s

    def __call__(self, tensor):
        tensor *= self.s
        return tensor

    def randomize_parameters(self):
        pass


class Resize(transforms.Resize):
    def randomize_parameters(self):
        pass


class Scale(transforms.Scale):
    def randomize_parameters(self):
        pass


class CenterCrop(transforms.CenterCrop):
    def randomize_parameters(self):
        pass


class LoopPadding(object):
    def __init__(self, size):
        self.size = size

    def __call__(self, frame_indices):
        out = frame_indices

        for index in out:
            if len(out) >= self.size:
                break
            out.append(index)

        return out


class SlidingWindow(object):
    def __init__(self, size, stride=0):
        self.size = size
        if stride == 0:
            self.stride = self.size
        else:
            self.stride = stride
        self.loop = LoopPadding(size)

    def __call__(self, frame_indices):
        out = []
        for begin_index in frame_indices[:: self.stride]:
            end_index = min(frame_indices[-1] + 1, begin_index + self.size)
            sample = list(range(begin_index, end_index))

            if len(sample) < self.size:
                out.append(self.loop(sample))
                break
            else:
                out.append(sample)

        return out


def get_normalize_method(mean, std, no_mean_norm, no_std_norm):
    if no_mean_norm:
        if no_std_norm:
            return Normalize([0, 0, 0], [1, 1, 1])
        else:
            return Normalize([0, 0, 0], std)
    else:
        if no_std_norm:
            return Normalize(mean, [1, 1, 1])
        else:
            return Normalize(mean, std)


class ImageLoaderPIL(object):
    def __call__(self, path):
        # open path as file to avoid ResourceWarning (https://github.com/python-pillow/Pillow/issues/835)
        with path.open("rb") as f:
            with Image.open(f) as img:
                return img.convert("RGB")


class VideoLoader(object):
    def __init__(self, image_name_formatter, image_loader=None):
        self.image_name_formatter = image_name_formatter
        if image_loader is None:
            self.image_loader = ImageLoaderPIL()
        else:
            self.image_loader = image_loader

    def __call__(self, video_path, frame_indices):
        video = []
        for i in frame_indices:
            image_path = video_path / self.image_name_formatter(i)
            if image_path.exists():
                video.append(self.image_loader(image_path))

        return video


def image_name_formatter(x):
    return f"image_{x:05d}.jpg"


class VideoDatasetMultiClips(VideoDataset):
    def __loading(self, path, video_frame_indices):
        clips = []
        segments = []
        for clip_frame_indices in video_frame_indices:
            clip = self.loader(path, clip_frame_indices)
            if self.spatial_transform is not None:
                self.spatial_transform.randomize_parameters()
                clip = [self.spatial_transform(img) for img in clip]
            clips.append(torch.stack(clip, 0).permute(1, 0, 2, 3))
            segments.append([min(clip_frame_indices), max(clip_frame_indices) + 1])

        return clips, segments

    def __getitem__(self, index):
        path = self.data[index]["video"]
        video_frame_indices = self.data[index]["frame_indices"]
        if self.temporal_transform is not None:
            video_frame_indices = self.temporal_transform(video_frame_indices)

        clips, segments = self.__loading(path, video_frame_indices)

        if isinstance(self.target_type, list):
            target = [self.data[index][t] for t in self.target_type]
        else:
            target = self.data[index][self.target_type]

        if "segment" in self.target_type:
            if isinstance(self.target_type, list):
                segment_index = self.target_type.index("segment")
                targets = []
                for s in segments:
                    targets.append(copy.deepcopy(target))
                    targets[-1][segment_index] = s
            else:
                targets = segments
        else:
            targets = [target for _ in range(len(segments))]
        return clips, targets


class TemporalCompose(object):
    def __init__(self, transforms):
        self.transforms = transforms

    def __call__(self, frame_indices):
        for i, t in enumerate(self.transforms):
            if isinstance(frame_indices[0], list):
                next_transforms = TemporalCompose(self.transforms[i:])
                dst_frame_indices = [
                    next_transforms(clip_frame_indices)
                    for clip_frame_indices in frame_indices
                ]

                return dst_frame_indices
            else:
                frame_indices = t(frame_indices)
        return frame_indices


class CalibData(mm.CalibDataInterface):
    def __init__(self, shape: mm.Dims, max_samples: int, img_dir: str):
        super().__init__()
        print(img_dir)
        assert os.path.isdir(img_dir)
        self.dir_path = img_dir
        self.data_paths_ = glob.glob(
            img_dir + "abseiling/3E7Jib8Yq5M_000118_000128/*.jpg"
        )
        self.shape_ = shape
        self.max_samples_ = min(max_samples, len(self.data_paths_))
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        self.dst_shape_ = (self.shape_.GetDimValue(2), self.shape_.GetDimValue(3))

        inference_crop = "center"
        mean = [0.4345, 0.4051, 0.3775]
        std = [0.2768, 0.2713, 0.2737]
        no_mean_norm = False
        no_std_norm = False
        sample_size = 112
        sample_duration = 16
        inference_stride = 16
        value_scale = 1
        inference_subset = "val"

        normalize = get_normalize_method(mean, std, no_mean_norm, no_std_norm)

        spatial_transform = [Resize(sample_size)]
        spatial_transform.append(CenterCrop(sample_size))
        spatial_transform.append(ToTensor())
        spatial_transform.extend([ScaleValue(value_scale), normalize])
        spatial_transform = Compose(spatial_transform)
        temporal_transform = []

        temporal_transform.append(SlidingWindow(sample_duration, inference_stride))

        temporal_transform = TemporalCompose(temporal_transform)

        loader = VideoLoader(image_name_formatter)
        video_path_formatter = (
            lambda root_path, label, video_id: root_path / label / video_id
        )

        video_path = Path(self.dir_path)
        annotation_path = Path(self.dir_path + "../../kinetics.json")
        subset = "validation"
        target_transform = None
        self.inference_data = VideoDatasetMultiClips(
            video_path,
            annotation_path,
            subset,
            spatial_transform=spatial_transform,
            temporal_transform=temporal_transform,
            target_transform=target_transform,
            video_loader=loader,
            video_path_formatter=video_path_formatter,
            target_type=["video_id", "segment"],
        )

    def get_shape(self):
        return self.shape_

    def get_data_type(self):
        return mm.DataType.FLOAT32

    def get_sample(self):
        return self.cur_sample_

    def preprocess_images(self, data_begin: int, data_end: int) -> np.ndarray:
        imgs = []
        k = 0
        count_img = 0
        for i in range(data_begin, data_end):
            if i < (len(self.inference_data.__getitem__(0)[0]) - 1):
                img = self.inference_data.__getitem__(0)[0][i].numpy()
                imgs.append(img[np.newaxis, :])
            elif i == (len(self.inference_data.__getitem__(0)[0]) - 1):
                count_img = i + 1
                while count_img < self.max_samples_:
                    for j in range(len(self.inference_data.__getitem__(k)[0])):
                        imgs.append(img[np.newaxis, :])
                        if (j + 1) == len(self.inference_data.__getitem__(k)[0]):
                            if count_img == self.max_samples_:
                                break
                            else:
                                k = k + 1
                        else:
                            if count_img == self.max_samples_:
                                break
                            else:
                                count_img = count_img + 1

        return np.ascontiguousarray(np.concatenate(tuple(imgs), axis=0))

    def next(self):
        batch_size = self.shape_.GetDimValue(0)
        data_begin = self.cur_data_index_
        data_end = data_begin + batch_size
        if data_end > self.max_samples_:
            return mm.Status(mm.Code.OUT_OF_RANGE, "Data end reached")
        self.cur_sample_ = self.preprocess_images(data_begin, data_end)
        self.cur_data_index_ = data_end
        return mm.Status.OK()

    def reset(self):
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        return mm.Status.OK()



import magicmind.python.runtime as mm
from torchvision import transforms
from tqdm import tqdm
from PIL import Image
import numpy as np
import argparse
import sys
import os

sys.path.append("../../../")
from utils import Record

from mm_runner import MMRunner
from logger import Logger

log = Logger()
IMAGE_HEIGHT = 224
IMAGE_WIDTH = 224


def imagenet_dataset(val_txt, image_file_path, count=-1):
    with open(val_txt, "r") as f:
        lines = f.readlines()
    log.info("%d pictures will be read." % len(lines))
    if len(lines) < count:
        log.info("infer pictures less than {}".format(count))
        return
    current_count = 0
    for line in lines:
        image_name, label = line.split(" ")
        image_path = os.path.join(image_file_path, image_name)
        img = Image.open(image_path)
        if img.mode != "RGB":
            img = img.convert("RGB")
        yield img, label.strip()
        current_count += 1
        if current_count >= count and count != -1:
            break


def pre_process(input_image, transpose):
    resize_h, resize_w = (256, 256)
    crop_h, crop_w = (IMAGE_HEIGHT, IMAGE_WIDTH)
    # cambricon-note: no need to minus mean and div vars here.
    # + because minus mean and div vars had been done
    # + by `insert_bn_before_firstnode` when generating mm model
    # + through passing means and vars to gen_model.py

    mean = [0.485, 0.456, 0.406]
    std = [0.229, 0.224, 0.225]
    normalize = transforms.Normalize(mean, std)

    # transforms.ToTensor(),
    # normalize,
    preprocess = transforms.Compose(
        [
            transforms.Resize(resize_h),
            transforms.CenterCrop(crop_h),
        ]
    )
    input_tensor = preprocess(input_image)
    # input_numpy = input_tensor.numpy()
    # if transpose:
    #    input_numpy = np.transpose(input_numpy, (1, 2, 0))
    # return input_numpy

    input_numpy = np.asarray(input_tensor)
    return input_numpy


def load_name(imagenet_label_path):
    name_map = {}
    with open(imagenet_label_path, "r") as f:
        lines = f.readlines()
    for line in lines:
        idx = line.split(" ")[0]
        name = " ".join(line.split(" ")[1:])
        name_map[int(idx)] = name.strip()
    return name_map


# -------- arg parse ------------
parser = argparse.ArgumentParser()
parser.add_argument(
    "--device_id", dest="device_id", type=int, default=0, help="device_id"
)
parser.add_argument(
    "--magicmind_model",
    dest="magicmind_model",
    type=str,
    default="../data/models/mm_model",
    help="save mm model to this path",
)
parser.add_argument(
    "--batch_size",
    dest="batch_size",
    type=int,
    default=1,
    help="batch_size used for infer",
)
parser.add_argument(
    "--image_dir",
    dest="image_dir",
    type=str,
    default="/path/to/modelzoo/datasets/imageNet2012/",
    help="imagenet val datasets",
)
parser.add_argument(
    "--image_num", dest="image_num", type=int, default=10, help="image number"
)
parser.add_argument(
    "--name_file",
    dest="name_file",
    type=str,
    default="datasets/imagenet/name.txt",
    help="imagenet name txt",
)
parser.add_argument(
    "--label_file",
    dest="label_file",
    type=str,
    default="/path/to/modelzoo/datasets/imageNet2012/labels.txt",
    help="imagenet val label txt",
)
parser.add_argument(
    "--result_file",
    dest="result_file",
    type=str,
    default="../data/output/infer_result.txt",
    help="result_file",
)
parser.add_argument(
    "--result_label_file",
    dest="result_label_file",
    type=str,
    default="../data/output/eval_labels.txt",
    help="result_label_file",
)
parser.add_argument(
    "--result_top1_file",
    dest="result_top1_file",
    type=str,
    default="../data/output/eval_result_1.txt",
    help="result_top1_file",
)
parser.add_argument(
    "--result_top5_file",
    dest="result_top5_file",
    type=str,
    default="../data/output/eval_result_5.txt",
    help="result_top5_file",
)

# -------- main ------------
if __name__ == "__main__":
    args = parser.parse_args()
    if not os.path.exists(args.magicmind_model):
        log.error(args.magicmind_model + " does not exist.")
        exit()

    # model 定义
    model = MMRunner(mm_file=args.magicmind_model, device_id=args.device_id)

    name_map = load_name(args.name_file)
    record = Record(args.result_file)
    result_label = Record(args.result_label_file)
    result_top1 = Record(args.result_top1_file)
    result_top5 = Record(args.result_top5_file)

    count = 0
    log.info("Start run ...")
    batch_size = args.batch_size
    image_num = args.image_num

    # cambricon-note:the default dtype is float32
    dataset = imagenet_dataset(
        val_txt=args.label_file, image_file_path=args.image_dir, count=image_num
    )
    rem_img_num = image_num % batch_size
    img_idx = 0
    batch_counter = 0
    imgs = np.empty([batch_size, IMAGE_HEIGHT, IMAGE_WIDTH, 3])
    labels = np.empty([batch_size])

    for img, label in tqdm(dataset, total=args.image_num):
        data = pre_process(img, transpose=True)
        infer_batch_size = (
            batch_size if img_idx < (image_num - rem_img_num) else rem_img_num
        )
        imgs[batch_counter % infer_batch_size, :, :, :] = data
        labels[batch_counter % infer_batch_size] = label
        batch_counter += 1
        img_idx += 1

        if batch_counter % infer_batch_size == 0:
            batch_counter = 0
            inputs = [imgs]

            # inference
            outputs = model(inputs)

            # post-process
            for pred_idx in range(infer_batch_size):
                pred = outputs[0][pred_idx]
                index = pred.argsort()[::-1]

                record.write("top5 result:", False)
                result_label.write("[%d]: %d" % (count, int(labels[pred_idx])), False)
                result_top1.write("[%d]: %d" % (count, index[0]), False)

                for i in range(5):
                    idx = index[i]
                    name = name_map[idx]
                    record.write("%d [%s]" % (i, name), False)
                    result_top5.write("[%d]: %d" % (count, idx), False)
                count += 1



import magicmind.python.runtime as mm
import argparse
import numpy as np
from tqdm import tqdm
import time
from transformers import AutoTokenizer,default_data_collator
from transformers.trainer_utils import EvalPrediction
from transformers.trainer_pt_utils import nested_concat,nested_truncate
from torch.utils.data import DataLoader, SequentialSampler
from datasets import load_dataset, load_metric

import os 

cur_dir = os.path.dirname(os.path.abspath(__file__))

# metrics
def compute_metrics(p: EvalPrediction):
    preds = p.predictions[0] if isinstance(p.predictions, tuple) else p.predictions
    preds = np.argmax(preds, axis=1)
    result = metric.compute(predictions=preds, references=p.label_ids)
    if len(result) > 1:
        result["combined_score"] = np.mean(list(result.values())).item()
    return result
    
# data preprocess
def preprocess_function(examples):
    result = tokenizer(examples['sentence1'], examples['sentence2'],padding='max_length', max_length=MAX_SEQ_LENGTH, truncation=True)
    return result

parser = argparse.ArgumentParser(description='mAP Calculation')
parser.add_argument('--magicmind_model', type=str, required=True)
parser.add_argument('--dev_id', help='The result data path', type=int)
parser.add_argument('--batch_size', help='batch_size', type=int, default=1)
parser.add_argument('--datasets_dir', type=str, default="")
parser.add_argument('--acc_result', help='acc_result', type=str, default="")
parser.add_argument('--test_nums', help='test_nums', type=int, default=-1)

if __name__ == "__main__":
    args = parser.parse_args()
    
    DEV_ID = args.dev_id
    MM_MODEL = args.magicmind_model
    MODEL_NAME = 'textattack/bert-base-uncased-MRPC'
    BATCH_SIZE = args.batch_size
    MAX_SEQ_LENGTH = 128
    
    # load glue mrpc dataset
    tokenizer = AutoTokenizer.from_pretrained(MODEL_NAME)
    raw_datasets = load_dataset('glue', 'mrpc', cache_dir=args.datasets_dir)
    raw_datasets = raw_datasets.map(preprocess_function, batched=True,
                                    load_from_cache_file=False,keep_in_memory=True,desc="Running tokenizer on dataset")
    eval_dataset = raw_datasets['validation']
    eval_sampler = SequentialSampler(eval_dataset)
    dataloader = DataLoader(eval_dataset, sampler=eval_sampler, batch_size=BATCH_SIZE,
                            collate_fn=default_data_collator, drop_last=True)

    metric = load_metric(os.path.join(cur_dir,'../export_model/glue.py'), 'mrpc')
        
    with mm.System() as mm_sys:  # 初始化系统
        dev_count = mm_sys.device_count()
        assert DEV_ID < dev_count
        # 打开MLU设备
        dev = mm.Device()
        dev.id = DEV_ID
        assert dev.active().ok()
        # 反序列化离线模型
        model = mm.Model()
        model.deserialize_from_file(args.magicmind_model)
        engine = model.create_i_engine()
        assert engine is not None
        # 创建Context
        context = engine.create_i_context()
        assert context is not None
        # 创建MLU任务队列
        queue = dev.create_queue()
        assert queue is not None
        # 创建输入tensor
        input_tensors = context.create_inputs()

        all_preds = None
        all_labels = None
        start_time = time.time()
        for step, inputs in tqdm(enumerate(dataloader), desc='Evaluating...'):
            if step > args.test_nums:break
            output_tensors = []
            input_tensors[0].from_numpy(inputs['input_ids'].numpy())
            input_tensors[1].from_numpy(inputs['attention_mask'].numpy())
            input_tensors[2].from_numpy(inputs['token_type_ids'].numpy())
            assert context.enqueue(input_tensors, output_tensors, queue).ok()
            assert queue.sync().ok()
            logits = output_tensors[0].asnumpy()
            all_preds = logits if all_preds is None else nested_concat(all_preds, logits)
            all_labels = inputs['labels'] if all_labels is None else nested_concat(all_labels, inputs['labels'])
        end_time = time.time()
        raw_datasets.cleanup_cache_files() 
        # eval preds
        num_samples = len(eval_dataset)
        all_preds  = nested_truncate(all_preds, num_samples)
        all_labels = nested_truncate(all_labels, num_samples)
        metrics = compute_metrics(EvalPrediction(predictions=all_preds, label_ids=all_labels))
        # prefix
        with open(args.acc_result,'w') as f:
            for key in list(metrics.keys()):
                if not key.startswith(f"eval_"):
                    metrics[f"{key}"] = metrics.pop(key)
                    f.writelines(key+":" + str(metrics[f"{key}"]) + '\n')
        f.close()
        throughput = num_samples / (end_time - start_time)
        print(metrics)
        print('throughput: %f samples per second' % throughput)



import magicmind.python.runtime as mm
from magicmind.python.runtime.parser import Parser
import argparse

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description = "model build")
    parser.add_argument('--precision', type=str,   default='force_float16', required=True ,help='Quant_mode')
    parser.add_argument('--batch_size', type=int,   default=1, required=True ,help='batch_size')
    parser.add_argument('--shape_mutable', type=str, default="", required=True ,help='shape_mutable')
    parser.add_argument('--onnx_model', type=str, default="", required=True ,help='onnx_model')
    parser.add_argument('--mm_model', type=str, default="", required=True ,help='mm_model')
    parser.add_argument('--input_size', type=int, default=224, required=True ,help='input_size')
    args = parser.parse_args()

    DEV_ID = 0
    BATCH_SIZE = args.batch_size
    ONNX_MODEL = args.onnx_model
    MM_MODEL = args.mm_model
    INPUT_SIZE=(args.input_size,args.input_size) #固定输入尺寸
    
    network = mm.Network()
    parser = Parser(mm.ModelKind.kOnnx)
    assert parser.parse(network, ONNX_MODEL).ok()
    assert network.get_input(0).set_dimension(mm.Dims((BATCH_SIZE, 3, INPUT_SIZE[0], INPUT_SIZE[1]))).ok()
    config = mm.BuilderConfig()
    precision_json_str = '{"precision_config" : { "precision_mode" : "%s" }}'%args.precision
    assert config.parse_from_string(precision_json_str).ok()
    assert config.parse_from_string("{\"opt_config\":{\"type64to32_conversion\":true}}").ok()
    assert config.parse_from_string("{\"opt_config\":{\"conv_scale_fold\":false}}").ok()
    if args.shape_mutable == 'true':
        assert config.parse_from_string('{"graph_shape_mutable": true}').ok()
        assert config.parse_from_string('{ \
        "dim_range": {  \
        "0": {  \
            "min": [1,3,224,224],  \
            "max": [16,3,1344,1920]  \
        }}}').ok()
    else:
        assert config.parse_from_string('{"graph_shape_mutable": false}').ok()
    # 指定设备
    assert config.parse_from_string('{"archs":[{"mtp_372": [2,6,8]}]}').ok()
    assert config.parse_from_string('{"precision_config": {"activation_quant_algo": "symmetric"}}').ok()
    assert config.parse_from_string('{"precision_config": {"weight_quant_granularity": "per_tensor"}}').ok()

    with mm.System() as mm_sys:
        dev_count = mm_sys.device_count()
        print("Device count: ", dev_count)
        if DEV_ID >= dev_count:
            print("Invalid DEV_ID set!")
            abort()
        dev = mm.Device()
        dev.id = DEV_ID
        assert dev.active().ok()
        builder = mm.Builder()
        assert builder is not None
        mm_model = builder.build_model("magicmind model", network, config)
        assert mm_model is not None
        assert mm_model.serialize_to_file(MM_MODEL).ok()



import os
import numpy as np
import cv2
from tqdm import tqdm
import argparse

def get_filelist(path):
    filelist = []
    file_len = len(path)
    for home, dirs, files in os.walk(path):
        for filename in files:
            file = os.path.join(home, filename)
            if file.endswith(('jpg')):
                label = file.split('/')[-1].split('_')[-1].split('.')[0]
                file = './' + file[file_len:] + ' ' + label
                print("file: ", file)
                filelist.append(file)
    return filelist

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--image_dir', required=True, help='path to dataset')
    args = parser.parse_args()
    filelist = get_filelist(args.image_dir)
    with open('annotation.txt', 'w') as f:
        for file in tqdm(filelist):
            f.write(file+'\n')
    f.close()



from transformers import AutoModelForSequenceClassification
import torch
from datasets import load_dataset
import sys
import os

cur_dir = os.path.dirname(os.path.abspath(__file__))

if __name__=="__main__":
    MODEL_NAME = 'textattack/bert-base-uncased-MRPC'
    GLUE_DATASETS_PATH = sys.argv[1]
    BATCH_SIZE = 1
    MAX_SEQ_LENGTH = 128
    # download dataset
    raw_datasets = load_dataset("glue", 'mrpc', cache_dir=os.path.join(GLUE_DATASETS_PATH))



import os
import numpy as np
import magicmind.python.runtime as mm
import argparse
import sys
import time
from tqdm import tqdm
sys.path.append("..")
from gen_model.preprocess import preprocess, imagenet_dataset

from utils import Record
from mm_runner import MMRunner
from logger import Logger
log = Logger()

def load_name(imagenet_label_path):
    name_map = {}
    with open(imagenet_label_path, "r") as f:
        lines = f.readlines()
    for line in lines:
        idx = line.split(" ")[0]
        name = " ".join(line.split(" ")[1:])
        name_map[int(idx)] = name.strip()
    return name_map

parser = argparse.ArgumentParser()
parser.add_argument("--device_id", "--device_id",  type=int, default=0, help="device_id")
parser.add_argument("--magicmind_model", "--magicmind_model", type=str, default="../data/models/renset50_onnx_model", help="save mm model to this path")
parser.add_argument("--image_dir", "--image_dir",  type=str, default="/path/to/modelzoo/datasets/imageNet2012/", help="imagenet val datasets")
parser.add_argument("--image_num", "--image_num",  type=int, default=10, help="image number")
parser.add_argument("--name_file", "--name_file",  type=str, default="datasets/imagenet/name.txt", help="imagenet name txt")
parser.add_argument("--label_file", "--label_file",  type=str, default="/path/to/modelzoo/datasets/imageNet2012/labels.txt", help="imagenet val label txt")
parser.add_argument("--result_file", "--result_file",  type=str, default="../data/images/output/infer_result.txt", help="result_file")
parser.add_argument("--result_label_file", "--result_label_file",  type=str, default="../data/images/output/eval_labels.txt", help="result_label_file")
parser.add_argument("--result_top1_file", "--result_top1_file",  type=str, default="../data/images/output/eval_result_1.txt", help="result_top1_file")
parser.add_argument("--result_top5_file", "--result_top5_file",  type=str, default="../data/images/output/eval_result_5.txt", help="result_top5_file")
parser.add_argument("--batch_size", "--batch_size",  type=int, default=1, help="batch_size")

if __name__ == "__main__":
    args = parser.parse_args()
    if not os.path.exists(args.magicmind_model):
        log.info("please generate magicmind model first!!!")
        exit()
        
    # model 定义    
    model = MMRunner(mm_file = args.magicmind_model,device_id = args.device_id)
    batch_size = args.batch_size 
    image_num = args.image_num

    name_map = load_name(args.name_file)
    record = Record(args.result_file)
    result_label = Record(args.result_label_file)
    result_top1 = Record(args.result_top1_file)
    result_top5 = Record(args.result_top5_file)

    dataset = imagenet_dataset(val_txt = args.label_file, image_file_path = args.image_dir, count = args.image_num)
    count = 0
    rem_img_num = image_num % batch_size
    img_idx = 0
    batch_counter = 0
    imgsz =  600
    imgs = np.empty([batch_size,imgsz,imgsz,3],dtype=np.float)
    labels = [ None for i in range(batch_size) ]
    
    for img, label in tqdm(dataset, total=args.image_num):
        # calc infer_batch_size
        infer_batch_size = batch_size if img_idx < (image_num-rem_img_num) else rem_img_num
        # pre-process
        imgs[ batch_counter % infer_batch_size,:,:,:] = preprocess(img, transpose=False, normalize=False)
        labels[ batch_counter % infer_batch_size] = label

        batch_counter += 1
        img_idx += 1

        if batch_counter % infer_batch_size == 0:
            batch_counter = 0 
            inputs = [imgs]
            # inference
            outputs = model(inputs)

            # post-process
            for pred_idx in range(infer_batch_size):
                index = outputs[0][pred_idx].argsort()[::-1]
                record.write("top5 result:", False)
                result_label.write("[%d]: %d"%(count, int(labels[pred_idx])), False)
                result_top1.write("[%d]: %d"%(count, index[0]), False)
                for i in range(5):
                    idx = index[i]
                    name = name_map[idx]
                    record.write("%d [%s]"%(i, name), False)
                    result_top5.write("[%d]: %d"%(count, idx), False)
                count += 1




import os

class Record:
    def __init__(self, filename):
        self.file = open(filename, "w")

    def write(self, line, _print = False):
        self.file.write(line + "\n")
        if _print:
            print(line)



# ------------------------------------------------------------------------
# Licensed under the Apache License, Version 2.0 (the "License")
# ------------------------------------------------------------------------

import argparse
import random

import torch

from models import build_model


def get_args_parser():
    parser = argparse.ArgumentParser('Set transformer detector', add_help=False)
    parser.add_argument('--lr', default=1e-4, type=float)
    parser.add_argument('--lr_backbone', default=1e-5, type=float)
    parser.add_argument('--batch_size', default=2, type=int)
    parser.add_argument('--weight_decay', default=1e-4, type=float)
    parser.add_argument('--epochs', default=250, type=int)
    parser.add_argument('--lr_drop', default=200, type=int)
    parser.add_argument('--clip_max_norm', default=0.1, type=float, help='gradient clipping max norm')

    # Backbone.
    parser.add_argument('--backbone', choices=['resnet50', 'resnet101'], required=True,
                        help="Name of the convolutional backbone to use")
    parser.add_argument('--position_embedding', default='sine', type=str, choices=('sine', 'learned'),
                        help="Type of positional embedding to use on top of the image features")

    # Transformer.
    parser.add_argument('--enc_layers', default=6, type=int,
                        help="Number of encoding layers in the transformer")
    parser.add_argument('--dec_layers', default=6, type=int,
                        help="Number of decoding layers in the transformer")
    parser.add_argument('--dim_feedforward', default=2048, type=int,
                        help="Intermediate size of the feedforward layers in the transformer blocks")
    parser.add_argument('--hidden_dim', default=256, type=int,
                        help="Size of the embeddings (dimension of the transformer)")
    parser.add_argument('--dropout', default=0.1, type=float,
                        help="Dropout applied in the transformer")
    parser.add_argument('--nheads', default=8, type=int,
                        help="Number of attention heads inside the transformer's attentions")
    parser.add_argument('--num_queries', default=100, type=int,
                        help="Number of query slots")
    parser.add_argument('--pre_norm', action='store_true')

    # Loss.
    parser.add_argument('--no_aux_loss', dest='aux_loss', action='store_false',
                        help="Disables auxiliary decoding losses (loss at each layer)")
    # Matcher.
    parser.add_argument('--set_cost_class', default=1, type=float,
                        help="Class coefficient in the matching cost")
    parser.add_argument('--set_cost_bbox', default=5, type=float,
                        help="L1 box coefficient in the matching cost")
    parser.add_argument('--set_cost_giou', default=2, type=float,
                        help="giou box coefficient in the matching cost")

    # Loss coefficients.
    parser.add_argument('--dice_loss_coef', default=1, type=float)
    parser.add_argument('--bbox_loss_coef', default=5, type=float)
    parser.add_argument('--giou_loss_coef', default=2, type=float)
    parser.add_argument('--eos_coef', default=0.02, type=float,
                        help="Relative classification weight of the no-object class")

    # Dataset parameters.
    parser.add_argument('--dataset_file', choices=['hico', 'vcoco', 'hoia'], required=True)

    parser.add_argument('--model_path', required=True,
                        help='Path of the model to evaluate.')
    parser.add_argument('--log_dir', default='./',
                        help='path where to save temporary files in test')
    parser.add_argument('--device', default='cpu',
                        help='device to use for training')
    parser.add_argument('--seed', default=42, type=int)
    parser.add_argument('--resume', default='', help='resume from checkpoint')
    parser.add_argument('--start_epoch', default=0, type=int, metavar='N',
                        help='start epoch')
    parser.add_argument('--num_workers', default=0, type=int)

    # Distributed training parameters.
    parser.add_argument('--world_size', default=1, type=int,
                        help='number of distributed processes')
    parser.add_argument('--dist_url', default='env://', help='url used to set up distributed training')

    # Visualization.
    parser.add_argument('--img_sheet', help='File containing image paths.')
    return parser



def load_model(model_path, args):
    checkpoint = torch.load(model_path, map_location='cpu')
    print('epoch:', checkpoint['epoch'])
    model, criterion = build_model(args)
    model.load_state_dict(checkpoint['model'])
    model.eval()
    return model



def export_onnx(model):

    onnx_model_name = "../data/models/hoitransformer.onnx"
    torch.onnx.export(model,         # model being run
         torch.randn(1,3,672,896),       # model input (or a tuple for multiple inputs)
         onnx_model_name,       # where to save the model
         export_params=True,  # store the trained parameter weights inside the model file
         opset_version=11,    # the ONNX version to export the model to
         do_constant_folding=True,  # whether to execute constant folding for optimization
         input_names = ['modelInput'],   # the model's input names
         output_names = ['modelOutput'], # the model's output names
         dynamic_axes={'modelInput' : {0 : 'batch_size', 1: 'channels', 2: 'h', 3: 'w'},    # variable length axes
             'modelOutput' : {0 : 'batch_size', 1: 'h', 2: 'w'}})
    return onnx_model_name

def main():
    parser = get_args_parser()
    args = parser.parse_args()
    model = load_model(model_path=args.model_path, args=args)
    onnx_model_name = export_onnx(model)
    print('done')
    print("onnx model name: ", onnx_model_name)


if __name__ == '__main__':
    main()



#! -*- coding: utf-8 -*-
# 测试有监督版RoFormer-Sim-FT的相似度效果

from bert4keras.backend import keras, K
from bert4keras.models import build_transformer_model
from bert4keras.tokenizers import Tokenizer
import tensorflow as tf

maxlen = 64
# bert配置
config_path = '../data/models/chinese_roformer-sim-char-ft_L-6_H-384_A-6/bert_config.json'
checkpoint_path = '../data/models/chinese_roformer-sim-char-ft_L-6_H-384_A-6/bert_model.ckpt'
dict_path = '../data/models/chinese_roformer-sim-char-ft_L-6_H-384_A-6/vocab.txt'
# 建立分词器
tokenizer = Tokenizer(dict_path, do_lower_case=True)  # 建立分词器

# 建立加载模型
bert = build_transformer_model(
    config_path,
    checkpoint_path,
    model='roformer',
    with_pool='linear',
    application='unilm',
    return_keras_model=False,
)

encoder = keras.models.Model(bert.model.inputs, bert.model.outputs[0])

out_nodes = []
for i in range(len(encoder.outputs)):
    out_nodes.append('output_' + str(i + 1))
    tf.identity(encoder.output[i],'output_' + str(i + 1))
sess = K.get_session()
# sess = tf.compat.v1.Session()
from tensorflow.python.framework import graph_util
init_graph = sess.graph.as_graph_def()
main_graph = graph_util.convert_variables_to_constants(sess,init_graph,out_nodes)

tf.compat.v1.graph_util.remove_training_nodes(
    main_graph, protected_nodes=None
)
with tf.gfile.GFile('../data/models/roformer.pb', 'wb') as fw:
    fw.write(main_graph.SerializeToString())

print("keras ckpt convert to tensorflow pb SUCCESSED!")


from setuptools import setup
from torch.utils.cpp_extension import BuildExtension, CppExtension

setup(
    name="torchvision_cpu_dcn",
    ext_modules=[
        CppExtension(
            "torchvision_cpu_dcn",
            ["cpu_dcn_kernel.cpp"],
            libraries=[],
        )
    ],
    cmdclass={"build_ext": BuildExtension.with_options(no_python_abi_suffix=True)},
)



import os
from PIL import Image
import numpy as np
import magicmind.python.runtime as mm
import argparse
import sys
sys.path.append("..")
from preprocess import preprocess, imagenet_dataset
sys.path.append("../../../")
from utils.utils import Record
def load_name(imagenet_label_path):
    name_map = {}
    with open(imagenet_label_path, "r") as f:
        lines = f.readlines()
    for line in lines:
        idx = line.split(" ")[0]
        name = " ".join(line.split(" ")[1:])
        name_map[int(idx)] = name.strip()
    return name_map

parser = argparse.ArgumentParser()
parser.add_argument("--device_id", "--device_id",  type=int, default=0, help="device_id")
parser.add_argument("--magicmind_model", "--magicmind_model", type=str, default="../data/models/renset50_onnx_model", help="save mm model to this path")
parser.add_argument("--image_dir", "--image_dir",  type=str, default="/path/to/modelzoo/datasets/imageNet2012/", help="imagenet val datasets")
parser.add_argument("--image_num", "--image_num",  type=int, default=10, help="image number")
parser.add_argument("--name_file", "--name_file",  type=str, default="datasets/imagenet/name.txt", help="imagenet name txt")
parser.add_argument("--label_file", "--label_file",  type=str, default="/path/to/modelzoo/datasets/imageNet2012/labels.txt", help="imagenet val label txt")
parser.add_argument("--result_file", "--result_file",  type=str, default="../data/images/output/infer_result.txt", help="result_file")
parser.add_argument("--result_label_file", "--result_label_file",  type=str, default="../data/images/output/eval_labels.txt", help="result_label_file")
parser.add_argument("--result_top1_file", "--result_top1_file",  type=str, default="../data/images/output/eval_result_1.txt", help="result_top1_file")
parser.add_argument("--result_top5_file", "--result_top5_file",  type=str, default="../data/images/output/eval_result_5.txt", help="result_top5_file")
parser.add_argument("--batch", "--batch",  type=int, default=1, help="batch_size")
if __name__ == "__main__":
    args = parser.parse_args()
    if not os.path.exists(args.magicmind_model):
        print("please generate magicmind model first!!!")
        exit()
    model = mm.Model()
    model.deserialize_from_file(args.magicmind_model)

    name_map = load_name(args.name_file)
    record = Record(args.result_file)
    result_label = Record(args.result_label_file)
    result_top1 = Record(args.result_top1_file)
    result_top5 = Record(args.result_top5_file)
    with mm.System():
        dev = mm.Device()
        dev.id = args.device_id
        econfig = mm.Model.EngineConfig()
        econfig.device_type = "MLU"
        engine = model.create_i_engine(econfig)
        assert engine != None, "Failed to create engine"
        context = engine.create_i_context()
        queue = dev.create_queue()
        assert queue != None

        inputs = context.create_inputs()
        outputs = []

        dataset = imagenet_dataset(val_txt = args.label_file, image_file_path = args.image_dir, count = args.image_num)
        count = 0
        for img, label in dataset:
            data = preprocess(img, transpose = False)
            data = np.expand_dims(data, 0)
            inputs[0].from_numpy(data)
            inputs[0].to(dev)
            
            assert context.enqueue(inputs, outputs, queue).ok()
            assert queue.sync().ok()

            index = outputs[0].asnumpy()[0].argsort()[::-1]
            record.write("top5 result:", False)
            result_label.write("[%d]: %d"%(count, int(label)), False)
            result_top1.write("[%d]: %d"%(count, index[0]), False)
            for i in range(5):
                idx = index[i]
                name = name_map[idx]
                record.write("%d [%s]"%(i, name), False)
                result_top5.write("[%d]: %d"%(count, idx), False)
            count += 1



import numpy as np
import os
from PIL import Image
import logging
import sys
sys.path.append(os.path.dirname(os.path.abspath(__file__)))
from operators import DecodeImage, ResizeImage, CropImage, NormalizeImage

def imagenet_dataset(
    val_txt,
    image_file_path,
    count=-1
):
    with open(val_txt, "r") as f:
        lines = f.readlines()
    logging.info("%d pictures will be read." % len(lines))
    current_count = 0
    for line in lines:
        image_name, label = line.split(" ")
        image_path = os.path.join(image_file_path, image_name)
        with open(image_path, 'rb') as f:
            img = f.read()
        yield img, label.strip()
        current_count += 1
        if current_count >= count and count != -1:
            break

def preprocess(input_image, dst_size=(600,600), transpose=True, normalize=True):
    resize_h, resize_w = (632, 632)
    crop_h, crop_w = dst_size
    mean = [0.485, 0.456, 0.406]
    std = [0.229, 0.224, 0.225]
    decodeimg = DecodeImage(to_rgb = True,to_np = True, channel_first = False)
    resizeimg = ResizeImage(size = None, resize_short = 632, interpolation=None, backend="cv2")
    cropimg =  CropImage(crop_h)
    normimg =  NormalizeImage(scale = 1/255, mean = mean, std =std, order = '')
    decodeout = decodeimg(input_image)
    resizeout = resizeimg(decodeout)
    cropout = cropimg(resizeout)
    if normalize:
        normout = normimg(cropout)
    else :
        normout = cropout

    input_numpy = normout
    if transpose:
        input_numpy = np.transpose(input_numpy, (2, 0, 1))
    return input_numpy

class Record:
    def __init__(self, filename):
        self.file = open(filename, "w+")

    def write(self, line, _print = False):
        self.file.write(line + "\n")
        if _print:
            print(line)

